from django.db import models, transaction
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from colorful.fields import RGBColorField

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import permission_document_view

from .events import (
    event_tag_attach, event_tag_created, event_tag_edited, event_tag_removed
)
from .html_widgets import widget_single_tag


@python_2_unicode_compatible
class Tag(models.Model):
    """
    This model represents a binary property that can be applied to a document.
    The tag can have a label and a color.
    """
    label = models.CharField(
        db_index=True, help_text=_(
            'A short text used as the tag name.'
        ), max_length=128, unique=True, verbose_name=_('Label')
    )
    color = RGBColorField(
        help_text=_('The RGB color values for the tag.'),
        verbose_name=_('Color')
    )
    documents = models.ManyToManyField(
        related_name='tags', to=Document, verbose_name=_('Documents')
    )

    class Meta:
        ordering = ('label',)
        verbose_name = _('Tag')
        verbose_name_plural = _('Tags')

    def __str__(self):
        return self.label

    def attach_to(self, document, user=None):
        """
        Attach a tag to a document and commit the corresponding event.
        """
        self.documents.add(document)
        event_tag_attach.commit(
            action_object=self, actor=user, target=document
        )

    def get_absolute_url(self):
        return reverse(
            viewname='tags:tag_document_list', kwargs={'tag_id': self.pk}
        )

    def get_document_count(self, user):
        """
        Return the numeric count of documents that have this tag attached.
        The count is filtered by access.
        """
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, queryset=self.documents,
            user=user
        )

        return queryset.count()

    def get_documents(self, user):
        """
        Return a filtered queryset documents that have this tag attached.
        """
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_view, queryset=self.documents.all(),
            user=user
        )

    def get_preview_widget(self):
        return widget_single_tag(tag=self)
    get_preview_widget.short_description = _('Preview')

    def remove_from(self, document, user=None):
        """
        Remove a tag from a document and commit the corresponding event.
        """
        self.documents.remove(document)
        event_tag_removed.commit(
            action_object=self, actor=user, target=document
        )

    def save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)
        created = not self.pk

        with transaction.atomic():
            result = super(Tag, self).save(*args, **kwargs)

            if created:
                event_tag_created.commit(actor=_user, target=self)
            else:
                event_tag_edited.commit(actor=_user, target=self)

            return result


class DocumentTag(Tag):
    class Meta:
        proxy = True
        verbose_name = _('Document tag')
        verbose_name_plural = _('Document tags')



from django import forms
from django.utils.html import conditional_escape


class TagFormWidget(forms.SelectMultiple):
    def __init__(self, *args, **kwargs):
        return super(TagFormWidget, self).__init__(*args, **kwargs)

    def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
        result = super(TagFormWidget, self).create_option(
            attrs=attrs, index=index,
            label='{}'.format(conditional_escape(label)), name=name,
            selected=selected, subindex=subindex, value=value
        )

        result['attrs']['data-color'] = self.choices.queryset.get(pk=value).color

        return result



from django.db import migrations
from django.utils.module_loading import import_string

STORAGE_PATH_UPDATE_MAP = {
    'mayan.apps.document_states.storages.storage_workflowimagecache': 'mayan.apps.document_states.storages.storage_workflow_image',
    'mayan.apps.documents.storages.storage_documentimagecache': 'mayan.apps.documents.storages.storage_document_image_cache'
}


def operation_purge_and_delete_caches(apps, schema_editor):
    Cache = apps.get_model(
        app_label='file_caching', model_name='Cache'
    )
    CachePartition = apps.get_model(
        app_label='file_caching', model_name='CachePartition'
    )
    CachePartitionFile = apps.get_model(
        app_label='file_caching', model_name='CachePartitionFile'
    )

    cache_storages = {}
    for cache in Cache.objects.using(schema_editor.connection.alias).all():
        cache_storages[cache.pk] = import_string(
            dotted_path=cache.storage_instance_path
        ).get_storage_instance()

    for cache_partition_file in CachePartitionFile.objects.using(schema_editor.connection.alias).all():
        cache_storages[
            cache_partition_file.partition.cache.pk
        ].delete(
            name='{}-{}'.format(
                cache_partition_file.partition.name,
                cache_partition_file.filename
            )
        )
        cache_partition_file.delete()

    for cache_partition in CachePartition.objects.using(schema_editor.connection.alias).all():
        cache_partition.delete()

    for cache in Cache.objects.using(schema_editor.connection.alias).all():
        cache.delete()


def operation_update_storage_paths(apps, schema_editor):
    Cache = apps.get_model(
        app_label='file_caching', model_name='Cache'
    )

    for cache in Cache.objects.using(schema_editor.connection.alias).all():
        cache.storage_instance_path = STORAGE_PATH_UPDATE_MAP.get(
            cache.storage_instance_path, cache.storage_instance_path
        )
        cache.save()


def operation_update_storage_paths_reverse(apps, schema_editor):
    storage_path_update_map_inverted = {
        value: key for key, value in STORAGE_PATH_UPDATE_MAP.items()
    }

    Cache = apps.get_model(
        app_label='file_caching', model_name='Cache'
    )

    for cache in Cache.objects.using(schema_editor.connection.alias).all():
        cache.storage_instance_path = storage_path_update_map_inverted.get(
            cache.storage_instance_path, cache.storage_instance_path
        )
        cache.save()


class Migration(migrations.Migration):
    dependencies = [
        ('file_caching', '0004_auto_20200309_0922'),
    ]
    operations = [
        migrations.RunPython(
            code=operation_update_storage_paths,
            reverse_code=operation_update_storage_paths_reverse
        ),
        migrations.RunPython(
            code=operation_purge_and_delete_caches,
            reverse_code=migrations.RunPython.noop
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_slow

queue_ocr = CeleryQueue(name='ocr', label=_('OCR'), worker=worker_slow)
queue_ocr.add_task_type(
    dotted_path='mayan.apps.ocr.tasks.task_do_ocr',
    label=_('Document version OCR')
)



import fcntl
from multiprocessing import Process
from pathlib import Path
import shutil

import mock

from django.core import mail
from django.utils.encoding import force_bytes, force_text

from django_celery_beat.models import PeriodicTask

from mayan.apps.common.serialization import yaml_dump
from mayan.apps.documents.models import Document
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import (
    TEST_COMPRESSED_DOCUMENT_PATH, TEST_NON_ASCII_DOCUMENT_FILENAME,
    TEST_NON_ASCII_DOCUMENT_PATH, TEST_NON_ASCII_COMPRESSED_DOCUMENT_PATH,
    TEST_SMALL_DOCUMENT_FILENAME, TEST_SMALL_DOCUMENT_PATH
)
from mayan.apps.metadata.models import MetadataType
from mayan.apps.storage.utils import mkdtemp

from ..literals import SOURCE_UNCOMPRESS_CHOICE_Y
from ..models.email_sources import EmailBaseModel, IMAPEmail, POP3Email
from ..models.scanner_sources import SaneScanner

from .literals import (
    TEST_EMAIL_ATTACHMENT_AND_INLINE, TEST_EMAIL_BASE64_FILENAME,
    TEST_EMAIL_BASE64_FILENAME_FROM, TEST_EMAIL_BASE64_FILENAME_SUBJECT,
    TEST_EMAIL_INLINE_IMAGE, TEST_EMAIL_NO_CONTENT_TYPE,
    TEST_EMAIL_NO_CONTENT_TYPE_STRING, TEST_EMAIL_ZERO_LENGTH_ATTACHMENT,
    TEST_WATCHFOLDER_SUBFOLDER
)
from .mixins import SourceTestMixin, WatchFolderTestMixin
from .mocks import MockIMAPServer, MockPOP3Mailbox


class CompressedUploadsTestCase(SourceTestMixin, GenericDocumentTestCase):
    auto_upload_test_document = False

    def test_upload_compressed_file(self):
        self.test_source.uncompress = SOURCE_UNCOMPRESS_CHOICE_Y
        self.test_source.save()

        with open(TEST_COMPRESSED_DOCUMENT_PATH, mode='rb') as file_object:
            self.test_source.handle_upload(
                document_type=self.test_document_type,
                file_object=file_object,
                expand=(
                    self.test_source.uncompress == SOURCE_UNCOMPRESS_CHOICE_Y
                )
            )

        self.assertEqual(Document.objects.count(), 2)
        self.assertTrue(
            'first document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )
        self.assertTrue(
            'second document.pdf' in Document.objects.values_list(
                'label', flat=True
            )
        )


class EmailBaseTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    def _create_email_source(self):
        self.source = EmailBaseModel(
            document_type=self.test_document_type,
            host='', username='', password='', store_body=True
        )

    def test_decode_email_base64_encoded_filename(self):
        """
        Test decoding of base64 encoded e-mail attachment filename.
        """
        self._create_email_source()
        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_BASE64_FILENAME
        )

        self.assertEqual(
            Document.objects.first().label, 'Ampelm\xe4nnchen.txt'
        )

    def test_decode_email_no_content_type(self):
        self._create_email_source()
        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_NO_CONTENT_TYPE
        )
        self.assertTrue(
            TEST_EMAIL_NO_CONTENT_TYPE_STRING in Document.objects.first().open().read()
        )

    def test_decode_email_zero_length_attachment(self):
        self._create_email_source()
        self.source.store_body = False
        self.source.save()

        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_ZERO_LENGTH_ATTACHMENT
        )

        self.assertEqual(Document.objects.count(), 0)

    def test_decode_email_with_inline_image(self):
        # Silence expected errors in other apps
        self._silence_logger(name='mayan.apps.converter.backends')

        self._create_email_source()
        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_INLINE_IMAGE
        )
        self.assertTrue(Document.objects.count(), 2)
        self.assertQuerysetEqual(
            ordered=False, qs=Document.objects.all(), values=(
                '<Document: test-01.png>', '<Document: email_body.html>'
            ),
        )

    def test_decode_email_with_attachment_and_inline_image(self):
        # Silence expected errors in other apps
        self._silence_logger(name='mayan.apps.converter.backends')

        self._create_email_source()
        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_ATTACHMENT_AND_INLINE
        )
        self.assertTrue(Document.objects.count(), 2)
        self.assertQuerysetEqual(
            ordered=False, qs=Document.objects.all(), values=(
                '<Document: test-01.png>', '<Document: email_body.html>',
            ),
        )

    def test_decode_email_and_store_from_and_subject_as_metadata(self):
        metadata_from = MetadataType.objects.create(name='from')
        metadata_subject = MetadataType.objects.create(name='subject')
        self.test_document_type.metadata.create(metadata_type=metadata_from)
        self.test_document_type.metadata.create(metadata_type=metadata_subject)

        self._create_email_source()
        self.source.from_metadata_type = metadata_from
        self.source.subject_metadata_type = metadata_subject
        self.source.save()

        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_BASE64_FILENAME
        )

        document = Document.objects.first()

        self.assertEqual(
            document.label, 'Ampelm\xe4nnchen.txt'
        )
        self.assertEqual(
            document.metadata.get(metadata_type=metadata_from).value,
            TEST_EMAIL_BASE64_FILENAME_FROM
        )
        self.assertEqual(
            document.metadata.get(metadata_type=metadata_subject).value,
            TEST_EMAIL_BASE64_FILENAME_SUBJECT
        )

    def test_document_upload_no_body(self):
        # Silence expected errors in other apps
        self._silence_logger(name='mayan.apps.converter.backends')

        self._create_email_source()
        self.source.store_body = False
        self.source.save()

        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_ATTACHMENT_AND_INLINE
        )

        # Only two attachments, no body document
        self.assertEqual(1, Document.objects.count())

    def test_document_upload_with_body(self):
        # Silence expected errors in other apps
        self._silence_logger(name='mayan.apps.converter.backends')

        self._create_email_source()

        EmailBaseModel.process_message(
            source=self.source, message_text=TEST_EMAIL_ATTACHMENT_AND_INLINE
        )

        # Only two attachments and a body document
        self.assertEqual(2, Document.objects.count())

    def test_metadata_yaml_attachment(self):
        TEST_METADATA_VALUE_1 = 'test value 1'
        TEST_METADATA_VALUE_2 = 'test value 2'

        test_metadata_type_1 = MetadataType.objects.create(
            name='test_metadata_type_1'
        )
        test_metadata_type_2 = MetadataType.objects.create(
            name='test_metadata_type_2'
        )
        self.test_document_type.metadata.create(
            metadata_type=test_metadata_type_1
        )
        self.test_document_type.metadata.create(
            metadata_type=test_metadata_type_2
        )

        test_metadata_yaml = yaml_dump(
            data={
                test_metadata_type_1.name: TEST_METADATA_VALUE_1,
                test_metadata_type_2.name: TEST_METADATA_VALUE_2,
            }
        )

        # Create email with a test attachment first, then the metadata.yaml
        # attachment
        with mail.get_connection(
            backend='django.core.mail.backends.locmem.EmailBackend'
        ) as connection:
            email_message = mail.EmailMultiAlternatives(
                body='test email body', connection=connection,
                subject='test email subject', to=['test@example.com'],
            )

            email_message.attach(
                filename='test_attachment',
                content='test_content',
            )

            email_message.attach(
                filename='metadata.yaml',
                content=test_metadata_yaml,
            )

            email_message.send()

        self._create_email_source()
        self.source.store_body = True
        self.source.save()

        EmailBaseModel.process_message(
            source=self.source, message_text=mail.outbox[0].message()
        )

        self.assertEqual(Document.objects.count(), 2)

        for document in Document.objects.all():
            self.assertEqual(
                document.metadata.get(metadata_type=test_metadata_type_1).value,
                TEST_METADATA_VALUE_1
            )
            self.assertEqual(
                document.metadata.get(metadata_type=test_metadata_type_2).value,
                TEST_METADATA_VALUE_2
            )


class IMAPSourceTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    @mock.patch('imaplib.IMAP4_SSL', autospec=True)
    def test_download_document(self, mock_imaplib):
        mock_imaplib.return_value = MockIMAPServer()
        self.source = IMAPEmail.objects.create(
            document_type=self.test_document_type, label='', host='',
            password='', username=''
        )

        self.source.check_source()
        self.assertEqual(
            Document.objects.first().label, 'Ampelm\xe4nnchen.txt'
        )


class IntervalSourceTestCase(WatchFolderTestMixin, GenericDocumentTestCase):
    auto_upload_test_document = False

    def setUp(self):
        super(IntervalSourceTestCase, self).setUp()
        self.temporary_directory = mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temporary_directory)
        super(IntervalSourceTestCase, self).tearDown()

    def test_periodic_task_create(self):
        periodic_task_count = PeriodicTask.objects.count()

        self._create_test_watchfolder()
        self.assertTrue(PeriodicTask.objects.count() > periodic_task_count)

    def test_periodic_task_delete(self):
        self._create_test_watchfolder()
        periodic_task_count = PeriodicTask.objects.count()

        self.test_document_type.delete()
        self.assertTrue(PeriodicTask.objects.count() < periodic_task_count)


class SANESourceTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    def _create_test_scanner_source(self):
        self.test_source = SaneScanner.objects.create(
            label='', device_name='test'
        )

    def test_command(self):
        self._create_test_scanner_source()
        file_object = self.test_source.execute_command(arguments=('-V',))
        self.assertTrue(force_bytes('sane') in file_object.read())

    def test_scan(self):
        self._create_test_scanner_source()

        file_object = self.test_source.get_upload_file_object(
            form_data={'document_type': self.test_document_type.pk}
        )
        self.assertTrue(file_object.size > 0)


class POP3SourceTestCase(GenericDocumentTestCase):
    auto_upload_test_document = False

    @mock.patch('poplib.POP3_SSL', autospec=True)
    def test_download_document(self, mock_poplib):
        mock_poplib.return_value = MockPOP3Mailbox()
        self.source = POP3Email.objects.create(
            document_type=self.test_document_type, label='', host='',
            password='', username=''
        )

        self.source.check_source()
        self.assertEqual(
            Document.objects.first().label, 'Ampelm\xe4nnchen.txt'
        )


class WatchFolderTestCase(WatchFolderTestMixin, GenericDocumentTestCase):
    auto_upload_test_document = False

    def setUp(self):
        super(WatchFolderTestCase, self).setUp()
        self.temporary_directory = mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temporary_directory)
        super(WatchFolderTestCase, self).tearDown()

    def test_subfolder_support_disabled(self):
        self._create_test_watchfolder()

        test_path = Path(self.temporary_directory)
        test_subfolder = test_path.joinpath(TEST_WATCHFOLDER_SUBFOLDER)
        test_subfolder.mkdir()

        shutil.copy(TEST_SMALL_DOCUMENT_PATH, force_text(test_subfolder))
        self.test_watch_folder.check_source()
        self.assertEqual(Document.objects.count(), 0)

    def test_subfolder_support_enabled(self):
        self._create_test_watchfolder()
        self.test_watch_folder.include_subdirectories = True
        self.test_watch_folder.save()

        test_path = Path(self.temporary_directory)
        test_subfolder = test_path.joinpath(TEST_WATCHFOLDER_SUBFOLDER)
        test_subfolder.mkdir()

        shutil.copy(TEST_SMALL_DOCUMENT_PATH, force_text(test_subfolder))
        self.test_watch_folder.check_source()
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()

        self.assertEqual(document.exists(), True)
        self.assertEqual(document.size, 17436)

        self.assertEqual(document.file_mimetype, 'image/png')
        self.assertEqual(document.file_mime_encoding, 'binary')
        self.assertEqual(document.label, TEST_SMALL_DOCUMENT_FILENAME)
        self.assertEqual(document.page_count, 1)

    def test_issue_gh_163(self):
        """
        Non-ASCII chars in document name failing in upload via watch folder
        gh-issue #163 https://github.com/mayan-edms/mayan-edms/issues/163
        """
        self._create_test_watchfolder()

        shutil.copy(TEST_NON_ASCII_DOCUMENT_PATH, self.temporary_directory)
        self.test_watch_folder.check_source()
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()

        self.assertEqual(document.exists(), True)
        self.assertEqual(document.size, 17436)

        self.assertEqual(document.file_mimetype, 'image/png')
        self.assertEqual(document.file_mime_encoding, 'binary')
        self.assertEqual(document.label, TEST_NON_ASCII_DOCUMENT_FILENAME)
        self.assertEqual(document.page_count, 1)

    def test_issue_gh_163_expanded(self):
        """
        Test Non-ASCII named documents inside Non-ASCII named compressed file
        """
        self._create_test_watchfolder()

        shutil.copy(
            TEST_NON_ASCII_COMPRESSED_DOCUMENT_PATH, self.temporary_directory
        )
        self.test_watch_folder.check_source()
        self.assertEqual(Document.objects.count(), 1)

        document = Document.objects.first()

        self.assertEqual(document.exists(), True)
        self.assertEqual(document.size, 17436)

        self.assertEqual(document.file_mimetype, 'image/png')
        self.assertEqual(document.file_mime_encoding, 'binary')
        self.assertEqual(document.label, TEST_NON_ASCII_DOCUMENT_FILENAME)
        self.assertEqual(document.page_count, 1)

    def test_locking_support(self):
        self._create_test_watchfolder()

        shutil.copy(
            TEST_SMALL_DOCUMENT_PATH, self.temporary_directory
        )

        path_test_file = Path(
            self.temporary_directory, TEST_SMALL_DOCUMENT_FILENAME
        )

        with path_test_file.open(mode='rb+') as file_object:
            fcntl.lockf(file_object, fcntl.LOCK_EX | fcntl.LOCK_NB)
            process = Process(target=self.test_watch_folder.check_source)
            process.start()
            process.join()

            self.assertEqual(Document.objects.count(), 0)



from django import forms
from django.utils.translation import ugettext_lazy as _

from ..fields import DocumentVersionField

from .document_forms import DocumentDownloadForm

__all__ = ('DocumentVersionDownloadForm', 'DocumentVersionPreviewForm',)


class DocumentVersionDownloadForm(DocumentDownloadForm):
    preserve_extension = forms.BooleanField(
        label=_('Preserve extension'), required=False,
        help_text=_(
            'Takes the file extension and moves it to the end of the '
            'filename allowing operating systems that rely on file '
            'extensions to open the downloaded document version correctly.'
        )
    )


class DocumentVersionPreviewForm(forms.Form):
    def __init__(self, *args, **kwargs):
        document_version = kwargs.pop('instance', None)
        super(DocumentVersionPreviewForm, self).__init__(*args, **kwargs)
        self.fields['document_version'].initial = document_version

    document_version = DocumentVersionField()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_key_delete, icon_key_detail, icon_key_download, icon_key_setup,
    icon_key_upload, icon_keyserver_search, icon_private_keys,
    icon_public_keys
)
from .permissions import (
    permission_key_delete, permission_key_download, permission_key_receive,
    permission_key_view, permission_key_upload, permission_keyserver_query
)

link_key_delete = Link(
    args=('resolved_object.pk',), icon_class=icon_key_delete,
    permissions=(permission_key_delete,), tags='dangerous', text=_('Delete'),
    view='django_gpg:key_delete'
)
link_key_detail = Link(
    args=('resolved_object.pk',), icon_class=icon_key_detail,
    permissions=(permission_key_view,), text=_('Details'),
    view='django_gpg:key_detail'
)
link_key_download = Link(
    args=('resolved_object.pk',), icon_class=icon_key_download,
    permissions=(permission_key_download,), text=_('Download'),
    view='django_gpg:key_download'
)
link_key_query = Link(
    icon_class=icon_keyserver_search,
    permissions=(permission_keyserver_query,), text=_('Query keyservers'),
    view='django_gpg:key_query'
)
link_key_receive = Link(
    args='object.key_id', keep_query=True,
    permissions=(permission_key_receive,), text=_('Import'),
    view='django_gpg:key_receive'
)
link_key_setup = Link(
    icon_class=icon_key_setup, permissions=(permission_key_view,),
    text=_('Key management'), view='django_gpg:key_public_list'
)
link_key_upload = Link(
    icon_class=icon_key_upload, permissions=(permission_key_upload,),
    text=_('Upload key'), view='django_gpg:key_upload'
)
link_private_keys = Link(
    icon_class=icon_private_keys, permissions=(permission_key_view,),
    text=_('Private keys'), view='django_gpg:key_private_list'
)
link_public_keys = Link(
    icon_class=icon_public_keys, permissions=(permission_key_view,),
    text=_('Public keys'), view='django_gpg:key_public_list'
)



from actstream.models import Action

from mayan.apps.common.tests.base import GenericViewTestCase

from ..events import event_tag_created, event_tag_edited
from ..models import Tag
from ..permissions import permission_tag_create, permission_tag_edit

from .mixins import TagTestMixin, TagViewTestMixin


class TagEventsTestCase(TagTestMixin, TagViewTestMixin, GenericViewTestCase):
    def test_tag_create_event_no_permissions(self):
        action_count = Action.objects.count()

        response = self._request_test_tag_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Action.objects.count(), action_count)

    def test_tag_create_event_with_permissions(self):
        self.grant_permission(permission=permission_tag_create)

        action_count = Action.objects.count()

        response = self._request_test_tag_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Action.objects.count(), action_count + 1)

        event = Action.objects.first()

        tag = Tag.objects.first()

        self.assertEqual(event.verb, event_tag_created.id)
        self.assertEqual(event.target, tag)
        self.assertEqual(event.actor, self._test_case_user)

    def test_tag_edit_event_no_permissions(self):
        self._create_test_tag()

        action_count = Action.objects.count()

        response = self._request_test_tag_edit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Action.objects.count(), action_count)

    def test_tag_edit_event_with_access(self):
        self._create_test_tag()

        self.grant_access(
            obj=self.test_tag, permission=permission_tag_edit
        )

        action_count = Action.objects.count()

        response = self._request_test_tag_edit_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Action.objects.count(), action_count + 1)

        event = Action.objects.first()

        self.assertEqual(event.verb, event_tag_edited.id)
        self.assertEqual(event.target, self.test_tag)
        self.assertEqual(event.actor, self._test_case_user)



from django.conf.urls import url

from .views import (
    SystemMailerLogEntryListView, MailDocumentLinkView, MailDocumentView,
    UserMailerBackendSelectionView, UserMailingCreateView,
    UserMailingDeleteView, UserMailingEditView, UserMailerLogEntryListView,
    UserMailerTestView, UserMailerListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/send/link/$',
        name='send_document_link', view=MailDocumentLinkView.as_view()
    ),
    url(
        regex=r'^documents/multiple/send/link/$',
        name='send_multiple_document_link',
        view=MailDocumentLinkView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/send/document/$',
        name='send_document', view=MailDocumentView.as_view()
    ),
    url(
        regex=r'^documents/multiple/send/document/$',
        name='send_multiple_document', view=MailDocumentView.as_view()
    ),
    url(
        regex=r'^system_mailer/log/$', name='system_mailer_error_log',
        view=SystemMailerLogEntryListView.as_view()
    ),
    url(
        regex=r'^user_mailers/backend/selection/$',
        name='user_mailer_backend_selection',
        view=UserMailerBackendSelectionView.as_view()
    ),
    url(
        regex=r'^user_mailers/(?P<class_path>[a-zA-Z0-9_.]+)/create/$',
        name='user_mailer_create', view=UserMailingCreateView.as_view()
    ),
    url(
        regex=r'^user_mailers/(?P<mailer_id>\d+)/delete/$',
        name='user_mailer_delete', view=UserMailingDeleteView.as_view()
    ),
    url(
        regex=r'^user_mailers/(?P<mailer_id>\d+)/edit/$',
        name='user_mailer_edit', view=UserMailingEditView.as_view()
    ),
    url(
        regex=r'^user_mailers/(?P<mailer_id>\d+)/log/$',
        name='user_mailer_log', view=UserMailerLogEntryListView.as_view()
    ),
    url(
        regex=r'^user_mailers/(?P<mailer_id>\d+)/test/$',
        name='user_mailer_test', view=UserMailerTestView.as_view()
    ),
    url(
        regex=r'^user_mailers/$', name='user_mailer_list',
        view=UserMailerListView.as_view()
    )
]



from rest_framework import serializers

from .models import Message


class MessageSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:message-detail'},
        }
        fields = (
            'end_datetime', 'enabled', 'label', 'message', 'start_datetime',
            'id', 'url'
        )
        model = Message



from django.shortcuts import get_object_or_404

from rest_framework import status
from rest_framework.response import Response

from mayan.apps.documents.models import Document, DocumentVersion
from mayan.apps.rest_api import generics

from .models import DocumentPageOCRContent, DocumentTypeSettings
from .permissions import (
    permission_document_type_ocr_setup, permission_ocr_content_view,
    permission_ocr_document,
)
from .serializers import (
    DocumentPageOCRContentSerializer, DocumentTypeOCRSettingsSerializer
)


class APIDocumentTypeOCRSettingsView(generics.RetrieveUpdateAPIView):
    """
    get: Return the document type OCR settings.
    patch: Set the document type OCR settings.
    put: Set the document type OCR settings.
    """
    lookup_field = 'document_type__pk'
    lookup_url_kwarg = 'pk'
    mayan_object_permissions = {
        'GET': (permission_document_type_ocr_setup,),
        'PATCH': (permission_document_type_ocr_setup,),
        'PUT': (permission_document_type_ocr_setup,)
    }
    queryset = DocumentTypeSettings.objects.all()
    serializer_class = DocumentTypeOCRSettingsSerializer


class APIDocumentOCRView(generics.GenericAPIView):
    """
    post: Submit a document for OCR.
    """
    mayan_object_permissions = {
        'POST': (permission_ocr_document,)
    }
    queryset = Document.objects.all()

    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def post(self, request, *args, **kwargs):
        self.get_object().submit_for_ocr()
        return Response(status=status.HTTP_202_ACCEPTED)


class APIDocumentVersionOCRView(generics.GenericAPIView):
    """
    post: Submit a document version for OCR.
    """
    lookup_url_kwarg = 'version_pk'
    mayan_object_permissions = {
        'POST': (permission_ocr_document,)
    }
    queryset = DocumentVersion.objects.all()

    def get_document(self):
        return get_object_or_404(klass=Document, pk=self.kwargs['document_pk'])

    def get_queryset(self):
        return self.get_document().versions.all()

    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def post(self, request, *args, **kwargs):
        self.get_object().submit_for_ocr()
        return Response(status=status.HTTP_202_ACCEPTED)


class APIDocumentPageOCRContentView(generics.RetrieveAPIView):
    """
    get: Returns the OCR content of the selected document page.
    """
    lookup_url_kwarg = 'page_pk'
    mayan_object_permissions = {
        'GET': (permission_ocr_content_view,),
    }
    serializer_class = DocumentPageOCRContentSerializer

    def get_document(self):
        return get_object_or_404(klass=Document, pk=self.kwargs['document_pk'])

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document().versions.all(),
            pk=self.kwargs['version_pk']
        )

    def get_queryset(self):
        return self.get_document_version().pages.all()

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()

        try:
            ocr_content = instance.ocr_content
        except DocumentPageOCRContent.DoesNotExist:
            ocr_content = DocumentPageOCRContent.objects.none()

        serializer = self.get_serializer(ocr_content)
        return Response(serializer.data)



from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext

from mayan.apps.common.generics import (
    MultipleObjectConfirmActionView, SingleObjectListView
)

from .models import Cache
from .permissions import permission_cache_purge, permission_cache_view

from .tasks import task_cache_purge


class CacheListView(SingleObjectListView):
    model = Cache
    object_permission = permission_cache_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'title': _('File caches list')
        }


class CachePurgeView(MultipleObjectConfirmActionView):
    model = Cache
    object_permission = permission_cache_purge
    pk_url_kwarg = 'cache_id'
    success_message_singular = '%(count)d cache submitted for purging.'
    success_message_plural = '%(count)d caches submitted for purging.'

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ungettext(
                singular='Submit the selected cache for purging?',
                plural='Submit the selected caches for purging?',
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result['object'] = queryset.first()

        return result

    def object_action(self, form, instance):
        task_cache_purge.apply_async(
            kwargs={'cache_id': instance.pk, 'user_id': self.request.user.pk}
        )



from django.utils.encoding import force_text

from ..permissions import (
    permission_document_download, permission_document_version_revert,
    permission_document_version_view,
)

from .base import GenericDocumentViewTestCase
from .literals import TEST_VERSION_COMMENT
from .mixins import DocumentVersionTestMixin, DocumentVersionViewTestMixin


class DocumentVersionViewTestCase(
    DocumentVersionTestMixin, DocumentVersionViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_version_download_view_no_permission(self):
        response = self._request_document_version_download()
        self.assertEqual(response.status_code, 404)

    def test_document_version_download_view_with_permission(self):
        # Set the expected_content_types for
        # common.tests.mixins.ContentTypeCheckMixin
        self.expected_content_types = (
            self.test_document.latest_version.mimetype,
        )

        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_document_version_download()
        self.assertEqual(response.status_code, 200)

        with self.test_document.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=force_text(self.test_document.latest_version),
                mime_type=self.test_document.latest_version.mimetype
            )

    def test_document_version_download_preserve_extension_view_with_permission(self):
        # Set the expected_content_types for
        # common.tests.mixins.ContentTypeCheckMixin
        self.expected_content_types = (
            self.test_document.latest_version.mimetype,
        )

        self.grant_access(
            obj=self.test_document, permission=permission_document_download
        )

        response = self._request_document_version_download(
            data={'preserve_extension': True}
        )
        self.assertEqual(response.status_code, 200)

        with self.test_document.open() as file_object:
            self.assert_download_response(
                response=response, content=file_object.read(),
                filename=self.test_document.latest_version.get_rendered_string(
                    preserve_extension=True
                ), mime_type=self.test_document.latest_version.mimetype
            )

    def test_document_version_list_no_permission(self):
        self._upload_new_version()

        response = self._request_document_version_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_version_list_with_access(self):
        self._upload_new_version()
        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_view
        )

        response = self._request_document_version_list_view()
        self.assertContains(
            response=response, status_code=200, text=TEST_VERSION_COMMENT
        )

    def test_document_version_revert_no_permission(self):
        first_version = self.test_document.latest_version
        self._upload_new_version()

        response = self._request_document_version_revert_view(
            document_version=first_version
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.versions.count(), 2)

    def test_document_version_revert_with_access(self):
        first_version = self.test_document.latest_version
        self._upload_new_version()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_revert
        )

        response = self._request_document_version_revert_view(
            document_version=first_version
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(self.test_document.versions.count(), 1)



from django.conf.urls import url

from .views import DocumentTemplateSandboxView

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/sandbox/$',
        name='document_template_sandbox',
        view=DocumentTemplateSandboxView.as_view()
    ),
]



from django import forms

from .widgets import DocumentPagesCarouselWidget, DocumentPageImageWidget


class DocumentField(forms.fields.Field):
    widget = DocumentPagesCarouselWidget


class DocumentPageField(forms.fields.Field):
    widget = DocumentPageImageWidget


class DocumentVersionField(forms.fields.Field):
    widget = DocumentPagesCarouselWidget



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('cabinets', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cabinet', name='label', field=models.CharField(
                help_text='A short text used to identify the cabinet.',
                max_length=128, verbose_name='Label'
            ),
        ),
    ]



from mayan.apps.appearance.classes import Icon

icon_event_types_subscriptions_list = Icon(
    driver_name='fontawesome', symbol='list-ol'
)
icon_events_for_object = Icon(driver_name='fontawesome', symbol='list-ol')
icon_events_list = Icon(driver_name='fontawesome', symbol='list-ol')
icon_object_event_types_user_subcriptions_list = Icon(
    driver_name='fontawesome', symbol='rss'
)
icon_user_notifications_list = Icon(
    driver_name='fontawesome', symbol='bell'
)



from django.conf.urls import url

from .api_views import APIMessageListView, APIMessageView
from .views import (
    MessageCreateView, MessageDeleteView, MessageEditView, MessageListView
)

urlpatterns = [
    url(regex=r'^messages/$', view=MessageListView.as_view(), name='message_list'),
    url(
        regex=r'^messages/create/$', view=MessageCreateView.as_view(),
        name='message_create'
    ),
    url(
        regex=r'^messages/(?P<pk>\d+)/delete/$', view=MessageDeleteView.as_view(),
        name='message_delete'
    ),
    url(
        regex=r'^messages/(?P<pk>\d+)/edit/$', view=MessageEditView.as_view(),
        name='message_edit'
    ),
]

api_urls = [
    url(
        regex=r'^messages/$', view=APIMessageListView.as_view(),
        name='message-list'
    ),
    url(
        regex=r'^messages/(?P<pk>[0-9]+)/$', view=APIMessageView.as_view(),
        name='message-detail'
    ),
]



import resource
import unittest

from django.test import override_settings, tag

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.common.tests.literals import EXCLUDE_TEST_TAG
from mayan.apps.documents.models import Document
from mayan.apps.documents.tests.base import DocumentTestMixin
from mayan.apps.documents.tests.literals import TEST_PDF_DOCUMENT_FILENAME

# This constant may need tweaking as document upload code path changes.
# The value is targeted at making the document upload process fail exactly
# during the MIME type detection phase. Different architectures may need
# different values.
MAXIMUM_HEAP_MEMORY = 140000000


@unittest.skip('This test should be used only in development.')
@tag('memory', EXCLUDE_TEST_TAG)
class MIMETypeTestCase(DocumentTestMixin, BaseTestCase):
    auto_upload_test_document = False
    test_document_filename = TEST_PDF_DOCUMENT_FILENAME

    def setUp(self):
        super(MIMETypeTestCase, self).setUp()
        resource.setrlimit(resource.RLIMIT_DATA, (MAXIMUM_HEAP_MEMORY, -1))

    def test_little_memory_full_file(self):
        with self.assertRaises(expected_exception=Exception):
            self._upload_test_document()

        self.assertEqual(Document.objects.count(), 0)

    @override_settings(MIMETYPE_FILE_READ_SIZE=1024)
    def test_little_memory_partial_file(self):
        self._upload_test_document()

        self.assertEqual(Document.objects.count(), 1)



from mayan.apps.appearance.classes import Icon
from mayan.apps.converter.icons import icon_transformations

icon_document_type = Icon(
    driver_name='fontawesome-layers', data=[
        {'class': 'fas fa-circle', 'transform': 'shrink-12 up-2'},
        {'class': 'fas fa-cog', 'transform': 'shrink-6 up-2', 'mask': 'fas fa-torah'}
    ], shadow_class='fas fa-torah'
)

icon_menu_documents = Icon(driver_name='fontawesome', symbol='book')

icon_dashboard_document_types = icon_document_type
icon_dashboard_documents_in_trash = Icon(
    driver_name='fontawesome', symbol='trash-alt'
)
icon_dashboard_pages_per_month = Icon(
    driver_name='fontawesome', symbol='copy'
)
icon_dashboard_new_documents_this_month = Icon(
    driver_name='fontawesome', symbol='calendar'
)
icon_dashboard_total_document = Icon(
    driver_name='fontawesome', symbol='book'
)
icon_document_quick_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_document = Icon(driver_name='fontawesome', symbol='book')
icon_document_list = icon_document
icon_document_page_count_update = Icon(
    driver_name='fontawesome', symbol='copy'
)
icon_document_preview = Icon(driver_name='fontawesome', symbol='eye')
icon_document_print = Icon(
    driver_name='fontawesome', symbol='print'
)
icon_document_properties = Icon(driver_name='fontawesome', symbol='info')
icon_document_trash_send = Icon(
    driver_name='fontawesome', symbol='trash-alt'
)

icon_document_transformations_clear = Icon(
    driver_name='fontawesome-dual',
    primary_symbol=icon_transformations.kwargs['symbol'],
    secondary_symbol='times'
)
icon_document_transformations_clone = Icon(
    driver_name='fontawesome-dual',
    primary_symbol=icon_transformations.kwargs['symbol'],
    secondary_symbol='arrow-right'
)

icon_document_type_change = icon_document_type

icon_favorite_document_add = Icon(
    driver_name='fontawesome-dual', primary_symbol='star',
    secondary_symbol='plus'
)
icon_document_image_loading = Icon(
    driver_name='fontawesomecss', css_classes='far fa-clock fa-2x'
)
icon_document_list = Icon(driver_name='fontawesome', symbol='book')
icon_document_list_deleted = Icon(
    driver_name='fontawesome', symbol='trash-alt'
)
icon_document_list_recent_access = Icon(
    driver_name='fontawesome', symbol='clock'
)
icon_document_return = icon_favorite_document_add = Icon(
    driver_name='fontawesome-dual', primary_symbol='book',
    secondary_symbol='chevron-left'
)

icon_favorite_document_list = Icon(driver_name='fontawesome', symbol='star')
icon_favorite_document_remove = Icon(
    driver_name='fontawesome-dual', primary_symbol='star',
    secondary_symbol='minus'
)

# Document pages

icon_document_page_disable = Icon(
    driver_name='fontawesomecss', css_classes='far fa-eye-slash'
)
icon_document_page_enable = Icon(
    driver_name='fontawesomecss', css_classes='far fa-eye'
)
icon_document_page_navigation_first = Icon(
    driver_name='fontawesome', symbol='step-backward'
)
icon_document_page_navigation_last = Icon(
    driver_name='fontawesome', symbol='step-forward'
)
icon_document_page_navigation_next = Icon(
    driver_name='fontawesome', symbol='arrow-right'
)
icon_document_page_navigation_previous = Icon(
    driver_name='fontawesome', symbol='arrow-left'
)
icon_document_page_return = icon_document_return
icon_document_page_rotate_left = Icon(
    driver_name='fontawesome', symbol='undo'
)
icon_document_page_rotate_right = Icon(
    driver_name='fontawesome', symbol='redo'
)
icon_document_page_view = Icon(
    driver_name='fontawesome', symbol='image'
)
icon_document_page_view_reset = Icon(
    driver_name='fontawesome', symbol='sync'
)
icon_document_page_zoom_in = Icon(
    driver_name='fontawesome', symbol='search-plus'
)
icon_document_page_zoom_out = Icon(
    driver_name='fontawesome', symbol='search-minus'
)
icon_document_pages = Icon(driver_name='fontawesome', symbol='copy')

# Document types

icon_document_type_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='book',
    secondary_symbol='plus'
)
icon_document_type_delete = Icon(driver_name='fontawesome', symbol='times')
icon_document_type_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_document_type_setup = icon_document_type


icon_document_type_list = icon_document_type

icon_document_type_filename = Icon(
    driver_name='fontawesome', symbol='keyboard'
)
icon_document_type_filename_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='keyboard',
    secondary_symbol='plus'
)
icon_document_type_filename_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_document_type_filename_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_document_type_filename_list = Icon(
    driver_name='fontawesome', symbol='keyboard'
)

icon_document_type_policies = Icon(driver_name='fontawesome', symbol='times')
icon_document_type_setup = icon_document_type

# Document versions

icon_document_version_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_version_list = Icon(
    driver_name='fontawesome', symbol='code-branch'
)
icon_document_version_return_document = icon_document_return
icon_document_version_return_list = Icon(
    driver_name='fontawesome', symbol='code-branch'
)
icon_document_version_view = Icon(
    driver_name='fontawesome', symbol='eye'
)
icon_document_version_revert = Icon(
    driver_name='fontawesome', symbol='undo'
)

icon_duplicated_document_list = Icon(
    driver_name='fontawesome', symbol='clone'
)
icon_duplicated_document_scan = Icon(
    driver_name='fontawesome-dual-classes', primary_class='fas fa-clone',
    secondary_class='far fa-eye'
)

icon_recent_added_document_list = Icon(
    driver_name='fontawesome', symbol='asterisk'
)

icon_trash_can_empty = Icon(
    driver_name='fontawesome-dual-classes', primary_class='fas fa-trash-alt',
    secondary_class='fas fa-minus'
)
icon_trashed_document_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_trashed_document_list = Icon(
    driver_name='fontawesome', symbol='trash-alt'
)
icon_trashed_document_multiple_delete = Icon(
    driver_name='fontawesome', symbol='trash-alt'
)
icon_trashed_document_multiple_restore = Icon(
    driver_name='fontawesome', symbol='recycle'
)
icon_trashed_document_restore = Icon(
    driver_name='fontawesome', symbol='recycle'
)



from django.db import migrations, models

import mayan.apps.document_signatures.models
import mayan.apps.storage.classes


class Migration(migrations.Migration):
    dependencies = [
        ('document_signatures', '0008_auto_20180429_0759'),
    ]

    operations = [
        migrations.AlterField(
            model_name='detachedsignature',
            name='signature_file',
            field=models.FileField(
                blank=True, null=True,
                storage=mayan.apps.storage.classes.FakeStorageSubclass(),
                upload_to=mayan.apps.document_signatures.models.upload_to,
                verbose_name='Signature file'
            ),
        ),
    ]



import logging

from django.utils.deprecation import MiddlewareMixin

from mayan.apps.permissions.classes import Permission
from mayan.apps.user_management.querysets import get_user_queryset

from ..events import (
    event_user_impersonation_ended, event_user_impersonation_started
)
from ..literals import (
    IMPERSONATE_VARIABLE_ID, IMPERSONATE_VARIABLE_DISABLE,
    IMPERSONATE_VARIABLE_PERMANENT
)
from ..permissions import permission_users_impersonate

logger = logging.getLogger(name=__name__)


class ImpersonateMiddleware(MiddlewareMixin):
    @staticmethod
    def get_user(pk, request):
        return get_user_queryset().exclude(pk=request.user.pk).get(
            pk=pk
        )

    @staticmethod
    def permission_check(user):
        return Permission.check_user_permissions(
            permissions=(permission_users_impersonate,), user=user
        )

    def process_request(self, request):
        impersonate_permanent_session = IMPERSONATE_VARIABLE_PERMANENT in request.session

        if not impersonate_permanent_session:
            if IMPERSONATE_VARIABLE_DISABLE in request.POST or IMPERSONATE_VARIABLE_DISABLE in request.GET:
                if IMPERSONATE_VARIABLE_ID in request.session:
                    user_impersonated_id = request.session[IMPERSONATE_VARIABLE_ID]
                    del request.session[IMPERSONATE_VARIABLE_ID]

                    user_new = ImpersonateMiddleware.get_user(
                        pk=user_impersonated_id, request=request
                    )
                    event_user_impersonation_ended.commit(
                        actor=request.user, target=user_new
                    )
            else:
                impersonate_id = request.GET.get(
                    IMPERSONATE_VARIABLE_ID, request.POST.get(
                        IMPERSONATE_VARIABLE_ID
                    )
                )
                impersonate_permanent = IMPERSONATE_VARIABLE_PERMANENT in request.GET or IMPERSONATE_VARIABLE_PERMANENT in request.POST

                if impersonate_id:
                    try:
                        impersonate_id = int(impersonate_id)
                    except ValueError:
                        logger.error(
                            'Unable to impersonate, invalid user ID value. %s',
                            impersonate_id
                        )
                    else:
                        if ImpersonateMiddleware.permission_check(user=request.user):
                            request.session[IMPERSONATE_VARIABLE_ID] = impersonate_id

                            user_new = ImpersonateMiddleware.get_user(
                                pk=request.session[IMPERSONATE_VARIABLE_ID],
                                request=request
                            )
                            event_user_impersonation_started.commit(
                                actor=request.user, target=user_new
                            )

                            if impersonate_permanent:
                                request.session[IMPERSONATE_VARIABLE_PERMANENT] = True

        if IMPERSONATE_VARIABLE_ID in request.session and ImpersonateMiddleware.permission_check(user=request.user):
            request.user = ImpersonateMiddleware.get_user(
                pk=request.session[IMPERSONATE_VARIABLE_ID], request=request
            )



from django import forms
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _

import mayan

from .widgets import TemplateWidget


class TemplateField(forms.CharField):
    widget = TemplateWidget

    def __init__(self, *args, **kwargs):
        self.model = kwargs.pop('model')
        self.model_variable = kwargs.pop('model_variable')
        self.initial_help_text = kwargs.pop('initial_help_text', '')
        super(TemplateField, self).__init__(*args, **kwargs)
        self.help_text = format_lazy(
            '{} {}', self.initial_help_text,
            _(
                'Use Django\'s default templating language '
                '(https://docs.djangoproject.com/en/%(django_version)s/ref/templates/builtins/). '
                'The {{ %(variable)s }} variable is available to the template.'
            ) % {
                'django_version': mayan.__django_version__,
                'variable': self.model_variable
            }
        )
        self.widget.attrs['model'] = self.model
        self.widget.attrs['data-model-variable'] = self.model_variable



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.contrib.auth.password_validation import validate_password
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from mayan.apps.acls.models import AccessControlList

from .permissions import permission_group_view


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    users_count = serializers.SerializerMethodField()

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:group-detail'}
        }
        fields = ('id', 'name', 'url', 'users_count')
        model = Group

    def get_users_count(self, instance):
        return instance.user_set.count()


class UserGroupListSerializer(serializers.Serializer):
    group_pk_list = serializers.CharField(
        help_text=_(
            'Comma separated list of group primary keys to assign this '
            'user to.'
        )
    )

    def create(self, validated_data):
        validated_data['user'].groups.clear()
        try:
            pk_list = validated_data['group_pk_list'].split(',')

            for group in Group.objects.filter(pk__in=pk_list):
                try:
                    AccessControlList.objects.check_access(
                        obj=group, permissions=(permission_group_view,),
                        user=self.context['request'].user
                    )
                except PermissionDenied:
                    pass
                else:
                    validated_data['user'].groups.add(group)
        except Exception as exception:
            raise ValidationError(exception)

        return {'group_pk_list': validated_data['group_pk_list']}


class UserSerializer(serializers.HyperlinkedModelSerializer):
    groups = GroupSerializer(many=True, read_only=True)
    groups_pk_list = serializers.CharField(
        help_text=_(
            'List of group primary keys to which to add the user.'
        ), required=False
    )
    password = serializers.CharField(
        required=False, style={'input_type': 'password'}, write_only=True
    )

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:user-detail'}
        }
        fields = (
            'first_name', 'date_joined', 'email', 'groups', 'groups_pk_list',
            'id', 'is_active', 'last_login', 'last_name', 'password', 'url',
            'username'
        )
        model = get_user_model()
        read_only_fields = ('groups', 'is_active', 'last_login', 'date_joined')
        write_only_fields = ('password', 'group_pk_list')

    def _add_groups(self, instance):
        instance.groups.add(
            *Group.objects.filter(pk__in=self.groups_pk_list.split(','))
        )

    def create(self, validated_data):
        self.groups_pk_list = validated_data.pop('groups_pk_list', '')
        password = validated_data.pop('password', None)
        instance = super(UserSerializer, self).create(validated_data)

        if password:
            instance.set_password(password)
            instance.save()

        if self.groups_pk_list:
            self._add_groups(instance=instance)

        return instance

    def update(self, instance, validated_data):
        self.groups_pk_list = validated_data.pop('groups_pk_list', '')

        if 'password' in validated_data:
            instance.set_password(validated_data['password'])
            validated_data.pop('password')

        instance = super(UserSerializer, self).update(instance, validated_data)

        if self.groups_pk_list:
            instance.groups.clear()
            self._add_groups(instance=instance)

        return instance

    def validate(self, data):
        if 'password' in data:
            validate_password(data['password'], self.instance)

        return data



from django.db import migrations


def operation_add_full_path(apps, schema_editor):
    WorkflowStateAction = apps.get_model(
        app_label='document_states', model_name='WorkflowStateAction'
    )

    for workflow_state_action in WorkflowStateAction.objects.using(schema_editor.connection.alias).all():
        workflow_state_action.action_path = 'mayan.apps.{}'.format(
            workflow_state_action.action_path
        )
        workflow_state_action.save()


def operation_remove_full_path(apps, schema_editor):
    WorkflowStateAction = apps.get_model(
        app_label='document_states', model_name='WorkflowStateAction'
    )

    for workflow_state_action in WorkflowStateAction.objects.using(schema_editor.connection.alias).all():
        workflow_state_action.action_path = workflow_state_action.action_path.replace(
            'mayan.apps.', ''
        )
        workflow_state_action.save()


class Migration(migrations.Migration):
    dependencies = [
        ('document_states', '0012_auto_20180823_2353'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_add_full_path,
            reverse_code=operation_remove_full_path
        )
    ]



import logging
import json

import requests

from django.utils.translation import ugettext_lazy as _

from .classes import WorkflowAction
from .exceptions import WorkflowStateActionError

logger = logging.getLogger(name=__name__)
DEFAULT_TIMEOUT = 4  # 4 seconds


class DocumentPropertiesEditAction(WorkflowAction):
    fields = {
        'document_label': {
            'label': _('Document label'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'The new label to be assigned to the document. Can be a '
                    'string or a template.'
                ), 'required': False
            },
        }, 'document_description': {
            'label': _('Document description'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'The new description to be assigned to the document. '
                    'Can be a string or a template.'
                ), 'required': False
            },
        },
    }
    field_order = ('document_label', 'document_description')
    label = _('Modify document properties')
    widgets = {
        'document_description': {
            'class': 'django.forms.widgets.Textarea', 'kwargs': {
                'attrs': {'rows': '10'},
            }
        }
    }

    def execute(self, context):
        self.document_label = self.form_data.get('document_label')
        self.document_description = self.form_data.get('document_description')
        new_label = None
        new_description = None

        if self.document_label:
            new_label = self.render_field(
                field_name='document_label', context=context
            )

        if self.document_description:
            new_description = self.render_field(
                field_name='document_description', context=context
            )

        if new_label or new_description:
            document = context['document']
            document.label = new_label or document.label
            document.description = new_description or document.description

            document.save()


class HTTPAction(WorkflowAction):
    fields = {
        'url': {
            'label': _('URL'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'Can be an IP address, a domain or a template. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ),
                'required': True
            },
        }, 'timeout': {
            'label': _('Timeout'),
            'class': 'django.forms.IntegerField', 'default': DEFAULT_TIMEOUT,
            'help_text': _(
                'Time in seconds to wait for a response. Can be a static '
                'value or a template. '
            ),
            'required': True

        }, 'payload': {
            'label': _('Payload'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'A JSON document to include in the request. Can also be '
                    'a template that return a JSON document. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ), 'required': False
            },
        }, 'username': {
            'label': _('Username'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'Username to use for making the request with HTTP Basic '
                    'Auth. Can be a static value or a template. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ), 'max_length': 192, 'required': False
            },
        }, 'password': {
            'label': _('Password'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'Password to use for making the request with HTTP Basic '
                    'Auth. Can be a static value or a template. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ), 'max_length': 192, 'required': False
            },
        }, 'method': {
            'label': _('Method'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'The HTTP method to use for the request. Typical choices '
                    'are OPTIONS, HEAD, POST, GET, PUT, PATCH, DELETE. '
                    'Can be a static value or a template that returns the '
                    'method text. Templates receive the workflow log entry '
                    'instance as part of their context via the '
                    'variable "entry_log". The "entry_log" in turn '
                    'provides the "workflow_instance", "datetime", '
                    '"transition", "user", and "comment" attributes.'
                ), 'required': True
            }
        }, 'headers': {
            'label': _('Headers'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'Headers to send with the HTTP request. Must be in JSON '
                    'format. Can be a static value or a template. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ), 'required': False
            },
        }
    }
    field_order = (
        'url', 'username', 'password', 'headers', 'timeout', 'method', 'payload'
    )
    label = _('Perform an HTTP request')
    previous_dotted_paths = (
        'mayan.apps.document_states.workflow_actions.HTTPPostAction',
    )
    widgets = {
        'payload': {
            'class': 'django.forms.widgets.Textarea', 'kwargs': {
                'attrs': {'rows': '10'},
            }
        },
        'headers': {
            'class': 'django.forms.widgets.Textarea', 'kwargs': {
                'attrs': {'rows': '10'},
            }
        }
    }

    def render_field_load(self, field_name, context):
        """
        Method to perform a template render and subsequent JSON load.
        """
        render_result = self.render_field(
            field_name=field_name, context=context
        ) or '{}'

        try:
            load_result = json.loads(s=render_result, strict=False)
        except Exception as exception:
            raise WorkflowStateActionError(
                _('%(field_name)s JSON error: %(exception)s') % {
                    'field_name': field_name, 'exception': exception
                }
            )

        logger.debug('load result: %s', load_result)

        return load_result

    def execute(self, context):
        headers = self.render_field_load(field_name='headers', context=context)
        method = self.render_field(field_name='method', context=context)
        password = self.render_field(field_name='password', context=context)
        payload = self.render_field_load(field_name='payload', context=context)
        timeout = self.render_field(field_name='timeout', context=context)
        url = self.render_field(field_name='url', context=context)
        username = self.render_field(field_name='username', context=context)

        if '.' in timeout:
            timeout = float(timeout)
        elif timeout:
            timeout = int(timeout)
        else:
            timeout = None

        authentication = None
        if username or password:
            authentication = requests.auth.HTTPBasicAuth(
                username=username, password=password
            )

        requests.request(
            method=method, url=url, json=payload, timeout=timeout,
            auth=authentication, headers=headers
        )



from .document_models import *  # NOQA
from .document_page_models import *  # NOQA
from .document_type_models import *  # NOQA
from .document_version_models import *  # NOQA
from .misc_models import *  # NOQA



import copy

from ..models import MetadataType

from .literals import (
    TEST_METADATA_TYPE_LABEL_EDITED, TEST_METADATA_TYPE_NAME_EDITED,
    TEST_METADATA_TYPES_FIXTURES, TEST_METADATA_VALUE_EDITED
)


class DocumentMetadataViewTestMixin(object):
    def _request_test_document_metadata_add_get_view(self):
        return self.get(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self.test_document.pk
            }, data={'metadata_type': self.test_metadata_type.pk}
        )

    def _request_test_document_metadata_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self.test_document.pk
            }, data={'metadata_type': self.test_metadata_type.pk}
        )

    def _request_test_document_metadata_multiple_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_add', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'metadata_type': [
                    metadata_type.pk for metadata_type in self.test_metadata_types
                ],
            }
        )

    def _request_test_document_metadata_edit_post_view(self):
        return self.post(
            viewname='metadata:metadata_edit', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'form-0-id': self.test_metadata_type.pk,
                'form-0-update': True,
                'form-0-value': TEST_METADATA_VALUE_EDITED,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }
        )

    def _request_test_document_metadata_list_view(self):
        return self.get(
            viewname='metadata:metadata_view', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_metadata_remove_get_view(self):
        return self.get(
            viewname='metadata:metadata_remove', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_metadata_remove_post_view(self, index=0):
        return self.post(
            viewname='metadata:metadata_remove',
            kwargs={'document_id': self.test_document.pk}, data={
                'form-0-id': self.test_metadata_types[index].pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }
        )

    def _request_test_document_multiple_metadata_add_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_add', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                ),
                'metadata_type': self.test_metadata_type.pk
            }
        )

    def _request_test_document_multiple_metadata_edit_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_edit', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                ),
                'form-0-id': self.test_metadata_type.pk,
                'form-0-update': True,
                'form-0-value': TEST_METADATA_VALUE_EDITED,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }
        )

    def _request_test_document_multiple_metadata_remove_post_view(self):
        return self.post(
            viewname='metadata:metadata_multiple_remove', data={
                'id_list': '{},{}'.format(
                    self.test_documents[0].pk, self.test_documents[1].pk
                ),
                'form-0-id': self.test_metadata_type.pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }
        )


class MetadataTypeAPIViewTestMixin(object):
    def setUp(self):
        super(MetadataTypeAPIViewTestMixin, self).setUp()
        self.test_metadata_types_fixtures_api_views = copy.copy(
            TEST_METADATA_TYPES_FIXTURES
        )

    def _request_test_metadata_type_create_view(self):
        return self.post(
            viewname='rest_api:metadatatype-list',
            data=self.test_metadata_types_fixtures_api_views.pop()
        )

    def _request_test_metadata_type_delete_view(self):
        return self.delete(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_pk': self.test_metadata_type.pk}
        )

    def _request_test_metadata_type_detail_view(self):
        return self.get(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_pk': self.test_metadata_type.pk}
        )

    def _request_test_metadata_type_edit_view_via_patch(self):
        return self.patch(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_pk': self.test_metadata_type.pk}, data={
                'label': '{} edited'.format(self.test_metadata_type.label),
                'name': '{}_edited'.format(self.test_metadata_type.name),
            }
        )

    def _request_test_metadata_type_edit_view_via_put(self):
        return self.put(
            viewname='rest_api:metadatatype-detail',
            kwargs={'metadata_type_pk': self.test_metadata_type.pk}, data={
                'label': '{} edited'.format(self.test_metadata_type.label),
                'name': '{}_edited'.format(self.test_metadata_type.name),
            }
        )

    def _request_test_metadata_type_list_view(self):
        return self.get(viewname='rest_api:metadatatype-list')


class MetadataTypeTestMixin(object):
    def setUp(self):
        super(MetadataTypeTestMixin, self).setUp()
        self.test_metadata_types_fixtures_models = copy.copy(
            TEST_METADATA_TYPES_FIXTURES
        )
        self.test_metadata_types = []

    def _get_test_metadata_type_queryset(self):
        return MetadataType.objects.filter(
            pk__in=[
                metadata_type.pk for metadata_type in self.test_metadata_types
            ]
        )

    def _create_test_metadata_type(self):
        self.test_metadata_type = MetadataType.objects.create(
            **self.test_metadata_types_fixtures_models.pop()
        )
        self.test_metadata_types.append(self.test_metadata_type)


class MetadataTypeViewTestMixin(object):
    def setUp(self):
        super(MetadataTypeViewTestMixin, self).setUp()
        self.test_metadata_types_fixtures_views = copy.copy(
            TEST_METADATA_TYPES_FIXTURES
        )

    def _request_test_document_type_relationship_edit_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:setup_document_type_metadata_types',
            kwargs={'document_type_id': self.test_document_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'required'
            }
        )

    def _request_test_metadata_type_create_view(self):
        return self.post(
            viewname='metadata:setup_metadata_type_create',
            data=self.test_metadata_types_fixtures_views.pop()
        )

    def _request_test_metadata_type_delete_view(self):
        return self.post(
            viewname='metadata:setup_metadata_type_delete', kwargs={
                'metadata_type_id': self.test_metadata_type.pk
            }
        )

    def _request_test_metadata_type_edit_view(self):
        return self.post(
            viewname='metadata:setup_metadata_type_edit', kwargs={
                'metadata_type_id': self.test_metadata_type.pk
            }, data={
                'label': TEST_METADATA_TYPE_LABEL_EDITED,
                'name': TEST_METADATA_TYPE_NAME_EDITED
            }
        )

    def _request_metadata_type_list_view(self):
        return self.get(
            viewname='metadata:setup_metadata_type_list',
        )

    def _request_test_metadata_type_relationship_edit_view(self):
        # This request assumes there is only one document type and
        # blindly sets the first form of the formset.

        return self.post(
            viewname='metadata:setup_metadata_type_document_types',
            kwargs={'metadata_type_id': self.test_metadata_type.pk}, data={
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-0-relationship_type': 'required'
            }
        )



import logging

from django.apps import apps
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.module_loading import import_string
from django.utils.translation import ugettext as _

from mayan.apps.common.literals import LIST_MODE_CHOICE_LIST

logger = logging.getLogger(name=__name__)


class SearchBackend(object):
    def search(self, global_and_search, search_model, query_string, user):
        raise NotImplementedError


class SearchField(object):
    """
    Search for terms in fields that directly belong to the parent SearchModel
    """
    def __init__(self, search_model, field, label, transformation_function=None):
        self.search_model = search_model
        self.field = field
        self.label = label
        self.transformation_function = transformation_function

    def get_full_name(self):
        return self.field

    def get_model(self):
        return self.search_model.model


@python_2_unicode_compatible
class SearchModel(object):
    _registry = {}

    @classmethod
    def all(cls):
        return sorted(list(cls._registry.values()), key=lambda x: x.label)

    @classmethod
    def as_choices(cls):
        return cls._registry

    @classmethod
    def get(cls, name):
        try:
            result = cls._registry[name]
        except KeyError:
            raise KeyError(_('No search model matching the query'))
        if not hasattr(result, 'serializer'):
            result.serializer = import_string(dotted_path=result.serializer_path)

        return result

    def __init__(
        self, app_label, model_name, serializer_path, label=None,
        list_mode=None, permission=None, queryset=None
    ):
        self.app_label = app_label
        self.list_mode = list_mode or LIST_MODE_CHOICE_LIST
        self.model_name = model_name
        self.search_fields = []
        self._model = None  # Lazy
        self._label = label
        self.serializer_path = serializer_path
        self.permission = permission
        self.queryset = queryset
        self.__class__._registry[self.get_full_name()] = self

    def __str__(self):
        return force_text(self.label)

    def add_model_field(self, *args, **kwargs):
        """
        Add a search field that directly belongs to the parent SearchModel
        """
        search_field = SearchField(self, *args, **kwargs)
        self.search_fields.append(search_field)

    def get_fields_simple_list(self):
        """
        Returns a list of the fields for the SearchModel
        """
        result = []
        for search_field in self.search_fields:
            result.append((search_field.get_full_name(), search_field.label))

        return result

    def get_full_name(self):
        return '%s.%s' % (self.app_label, self.model_name)

    def get_queryset(self):
        if self.queryset:
            return self.queryset()
        else:
            return self.model.objects.all()

    def get_search_field(self, full_name):
        try:
            return self.search_fields[full_name]
        except KeyError:
            raise KeyError('No search field named: %s' % full_name)

    @property
    def label(self):
        if not self._label:
            self._label = self.model._meta.verbose_name
        return self._label

    @property
    def model(self):
        if not self._model:
            self._model = apps.get_model(self.app_label, self.model_name)
        return self._model

    @property
    def pk(self):
        return self.get_full_name()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import icon_cache_purge, icon_file_caching
from .permissions import permission_cache_purge, permission_cache_view

link_caches_list = Link(
    icon_class=icon_file_caching, permissions=(permission_cache_view,),
    text=_('File caches'), view='file_caching:cache_list'
)
link_cache_purge = Link(
    icon_class=icon_cache_purge, kwargs={'cache_id': 'resolved_object.id'},
    permissions=(permission_cache_purge,), text=_('Purge cache'),
    view='file_caching:cache_purge'
)
link_cache_multiple_purge = Link(
    icon_class=icon_cache_purge, text=_('Purge cache'),
    view='file_caching:cache_multiple_purge'
)



# Generated by Django 2.2.10 on 2020-03-11 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0019_workflowtransition_condition'),
    ]

    operations = [
        migrations.AddField(
            model_name='workflowstateaction',
            name='condition',
            field=models.TextField(blank=True, help_text="The condition that will determine if this state action is executed or not. The condition is evaluated against the workflow instance. Conditions that return None or an empty string ('') are considered to be logical false, any other value is considered to be the logical true.", verbose_name='Condition'),
        ),
        migrations.AlterField(
            model_name='workflowtransition',
            name='condition',
            field=models.TextField(blank=True, help_text="The condition that will determine if this transition is enabled or not. The condition is evaluated against the workflow instance. Conditions that return None or an empty string ('') are considered to be logical false, any other value is considered to be the logical true.", verbose_name='Condition'),
        ),
    ]



from mayan.apps.acls.classes import ModelPermission
from mayan.apps.acls.permissions import permission_acl_edit
from mayan.apps.acls.tests.mixins import (
    ACLTestMixin, AccessControlListViewTestMixin
)
from mayan.apps.common.tests.base import BaseTestCase

from ..models import DocumentType
from ..permissions import permission_document_view

from .base import GenericDocumentViewTestCase


class DocumentTypeACLPermissionsTestCase(BaseTestCase):
    def test_document_type_permission_test(self):
        result = ModelPermission.get_for_class(klass=DocumentType)
        self.assertTrue(permission_document_view in result)


class DocumentTypeACLPermissionsViewTestCase(
    ACLTestMixin, AccessControlListViewTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_document_type_acl_permission_view_with_access(self):
        self.test_object = self.test_document_type
        self._create_test_acl()
        self.grant_access(
            obj=self.test_object, permission=permission_acl_edit
        )

        response = self._request_test_acl_permission_list_get_view()
        self.assertContains(
            response=response, text=permission_document_view.label,
            status_code=200
        )



import logging

from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import (
    menu_multi_item, menu_object, menu_tools, menu_user
)
from mayan.apps.navigation.classes import Separator

from .links import (
    link_logout, link_password_change, link_user_impersonate_start,
    link_user_multiple_set_password, link_user_set_password
)

logger = logging.getLogger(name=__name__)


class AuthenticationApp(MayanAppConfig):
    app_namespace = 'authentication'
    app_url = 'authentication'
    has_tests = True
    name = 'mayan.apps.authentication'
    verbose_name = _('Authentication')

    def ready(self):
        super(AuthenticationApp, self).ready()

        User = get_user_model()

        menu_multi_item.bind_links(
            links=(link_user_multiple_set_password,),
            sources=('user_management:user_list',)
        )

        menu_object.bind_links(
            links=(link_user_set_password,), sources=(User,)
        )

        menu_tools.bind_links(
            links=(link_user_impersonate_start,)
        )

        menu_user.bind_links(
            links=(
                Separator(), link_password_change, link_logout
            ), position=99
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_search, icon_search_advanced, icon_search_again,
)

link_search = Link(
    args='search_model.get_full_name', icon_class=icon_search,
    text=_('Search'), view='search:search'
)
link_search_advanced = Link(
    args='search_model.get_full_name', icon_class=icon_search_advanced,
    text=_('Advanced search'), view='search:search_advanced'
)
link_search_again = Link(
    args='search_model.get_full_name', icon_class=icon_search_again,
    keep_query=True, text=_('Search again'), view='search:search_again'
)



from mayan.apps.common.tests.base import GenericViewTestCase

from ..permissions import permission_workflow_view

from .mixins import (
    WorkflowRuntimeProxyStateViewTestMixin,
    WorkflowRuntimeProxyViewTestMixin, WorkflowTestMixin
)


class WorkflowRuntimeProxyViewTestCase(
    WorkflowRuntimeProxyViewTestMixin, WorkflowTestMixin, GenericViewTestCase
):
    def setUp(self):
        super(WorkflowRuntimeProxyViewTestCase, self).setUp()
        self._create_test_workflow()

    def test_workflow_runtime_proxy_document_list_view_no_permission(self):
        response = self._request_test_workflow_runtime_proxy_document_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow.label, status_code=404
        )

    def test_workflow_runtime_proxy_document_list_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_runtime_proxy_document_list_view()
        self.assertContains(
            response=response, text=self.test_workflow.label, status_code=200
        )

    def test_workflow_runtime_proxy_list_view_no_permission(self):
        response = self._request_test_workflow_runtime_proxy_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow.label, status_code=200
        )

    def test_workflow_runtime_proxy_list_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_runtime_proxy_list_view()
        self.assertContains(
            response=response, text=self.test_workflow.label, status_code=200
        )


class WorkflowRuntimeProxyStateViewTestCase(
    WorkflowRuntimeProxyStateViewTestMixin, WorkflowTestMixin,
    GenericViewTestCase
):
    def setUp(self):
        super(WorkflowRuntimeProxyStateViewTestCase, self).setUp()
        self._create_test_workflow()
        self._create_test_workflow_states()

    def test_workflow_runtime_proxy_state_document_list_view_no_permission(self):
        response = self._request_test_workflow_runtime_proxy_state_document_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow_state_1.label,
            status_code=404
        )

    def test_workflow_runtime_proxy_state_document_list_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_runtime_proxy_state_document_list_view()
        self.assertContains(
            response=response, text=self.test_workflow_state_1.label,
            status_code=200
        )

    def test_workflow_runtime_proxy_state_list_view_no_permission(self):
        response = self._request_test_workflow_runtime_proxy_state_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow_state_1.label,
            status_code=404
        )

    def test_workflow_runtime_proxy_state_list_view_with_access(self):
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_runtime_proxy_state_list_view()
        self.assertContains(
            response=response, text=self.test_workflow_state_1.label,
            status_code=200
        )



from django.dispatch import Signal

post_version_upload = Signal(providing_args=('instance',), use_caching=True)
post_document_type_change = Signal(
    providing_args=('instance',), use_caching=True
)
post_document_created = Signal(providing_args=('instance',), use_caching=True)
post_initial_document_type = Signal(
    providing_args=('instance',), use_caching=True
)



import json

from ..models import UserMailer

from .literals import (
    TEST_EMAIL_ADDRESS,
    TEST_EMAIL_FROM_ADDRESS,
    TEST_USER_MAILER_BACKEND_PATH,
    TEST_USER_MAILER_LABEL
)


class MailerTestMixin(object):
    def _create_test_user_mailer(self):
        self.test_user_mailer = UserMailer.objects.create(
            default=True,
            enabled=True,
            label=TEST_USER_MAILER_LABEL,
            backend_path=TEST_USER_MAILER_BACKEND_PATH,
            backend_data=json.dumps(
                obj={
                    'from': TEST_EMAIL_FROM_ADDRESS
                }
            )
        )


class MailerViewTestMixin(object):
    def _request_test_document_link_send_view(self):
        return self.post(
            viewname='mailer:send_document_link', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'email': getattr(
                    self, 'test_email_address', TEST_EMAIL_ADDRESS
                ),
                'user_mailer': self.test_user_mailer.pk
            },
        )

    def _request_test_document_send_view(self):
        return self.post(
            viewname='mailer:send_document', kwargs={
                'document_id': self.test_document.pk
            }, data={
                'email': getattr(
                    self, 'test_email_address', TEST_EMAIL_ADDRESS
                ),
                'user_mailer': self.test_user_mailer.pk
            }
        )

    def _request_test_user_mailer_create_view(self):
        return self.post(
            viewname='mailer:user_mailer_create', kwargs={
                'class_path': TEST_USER_MAILER_BACKEND_PATH
            }, data={
                'default': True,
                'enabled': True,
                'label': TEST_USER_MAILER_LABEL,
            }
        )

    def _request_test_user_mailer_delete_view(self):
        return self.post(
            viewname='mailer:user_mailer_delete', kwargs={
                'mailer_id': self.test_user_mailer.pk
            }
        )

    def _request_test_user_mailer_list_view(self):
        return self.get(
            viewname='mailer:user_mailer_list'
        )

    def _request_test_user_mailer_log_entry_view(self):
        return self.get(
            viewname='mailer:user_mailer_log', kwargs={
                'mailer_id': self.test_user_mailer.pk
            }
        )

    def _request_test_user_mailer_test_view(self):
        return self.post(
            viewname='mailer:user_mailer_test', kwargs={
                'mailer_id': self.test_user_mailer.pk
            }, data={
                'email': getattr(
                    self, 'test_email_address', TEST_EMAIL_ADDRESS
                )
            }
        )



from django.apps import apps
from django.db import models


class MetadataTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

    def get_for_document(self, document):
        return self.filter(
            pk__in=document.metadata.values_list(
                'metadata_type', flat=True
            )
        )

    def get_for_document_type(self, document_type):
        return self.filter(
            pk__in=document_type.metadata.values_list(
                'metadata_type', flat=True
            )
        )


class DocumentTypeMetadataTypeManager(models.Manager):
    def get_by_natural_key(self, document_natural_key, metadata_type_natural_key):
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        MetadataType = apps.get_model(
            app_label='metadata', model_name='MetadataType'
        )
        try:
            document = Document.objects.get_by_natural_key(document_natural_key)
        except Document.DoesNotExist:
            raise self.model.DoesNotExist
        else:
            try:
                metadata_type = MetadataType.objects.get_by_natural_key(metadata_type_natural_key)
            except MetadataType.DoesNotExist:
                raise self.model.DoesNotExist

        return self.get(document__pk=document.pk, metadata_type__pk=metadata_type.pk)

    def get_metadata_types_for(self, document_type):
        DocumentType = apps.get_model(
            app_label='metadata', model_name='MetadataType'
        )

        return DocumentType.objects.filter(
            pk__in=self.filter(
                document_type=document_type
            ).values_list('metadata_type__pk', flat=True)
        )



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .permissions import (
    permission_document_version_sign_detached,
    permission_document_version_sign_embedded,
    permission_document_version_signature_delete,
    permission_document_version_signature_download,
    permission_document_version_signature_upload,
    permission_document_version_signature_verify,
    permission_document_version_signature_view
)


def is_detached_signature(context):
    SignatureBaseModel = apps.get_model(
        app_label='document_signatures', model_name='SignatureBaseModel'
    )

    return SignatureBaseModel.objects.select_subclasses().get(
        pk=context['object'].pk
    ).is_detached


link_document_version_all_signature_verify = Link(
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_all_signature_verify',
    permissions=(permission_document_version_signature_verify,),
    text=_('Verify all documents'),
    view='signatures:all_document_version_signature_verify',
)
link_document_signature_list = Link(
    args='resolved_object.latest_version.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_signature_list',
    permissions=(permission_document_version_signature_view,),
    text=_('Signatures'), view='signatures:document_version_signature_list',
)
link_document_version_signature_delete = Link(
    args='resolved_object.pk', condition=is_detached_signature,
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_delete',
    permissions=(permission_document_version_signature_delete,),
    tags='dangerous', text=_('Delete'),
    view='signatures:document_version_signature_delete',
)
link_document_version_signature_detached_create = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_detached_create',
    permissions=(permission_document_version_sign_detached,),
    text=_('Sign detached'),
    view='signatures:document_version_signature_detached_create'
)
link_document_version_signature_details = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_details',
    permissions=(permission_document_version_signature_view,),
    text=_('Details'), view='signatures:document_version_signature_details',
)
link_document_version_signature_download = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_download',
    condition=is_detached_signature,
    permissions=(permission_document_version_signature_download,),
    text=_('Download'), view='signatures:document_version_signature_download'
)
link_document_version_signature_embedded_create = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_embedded_create',
    permissions=(permission_document_version_sign_embedded,),
    text=_('Sign embedded'),
    view='signatures:document_version_signature_embedded_create'
)
link_document_version_signature_list = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_list',
    permissions=(permission_document_version_signature_view,),
    text=_('Signatures'), view='signatures:document_version_signature_list'
)
link_document_version_signature_upload = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.document_signatures.icons.icon_document_version_signature_upload',
    permissions=(permission_document_version_signature_upload,),
    text=_('Upload signature'),
    view='signatures:document_version_signature_upload'
)



import logging

from furl import furl

from django.db import models
from django.urls import reverse
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from mayan.apps.converter.literals import DEFAULT_ZOOM_LEVEL, DEFAULT_ROTATION

from mayan.apps.converter.models import LayerTransformation
from mayan.apps.converter.transformations import (
    BaseTransformation, TransformationResize, TransformationRotate,
    TransformationZoom
)
from mayan.apps.converter.utils import get_converter_class

from ..managers import DocumentPageManager
from ..settings import (
    setting_disable_base_image_cache, setting_disable_transformed_image_cache,
    setting_display_width, setting_display_height, setting_zoom_max_level,
    setting_zoom_min_level
)

from .document_version_models import DocumentVersion

__all__ = ('DocumentPage', 'DocumentPageResult')
logger = logging.getLogger(name=__name__)


@python_2_unicode_compatible
class DocumentPage(models.Model):
    """
    Model that describes a document version page
    """
    document_version = models.ForeignKey(
        on_delete=models.CASCADE, related_name='version_pages', to=DocumentVersion,
        verbose_name=_('Document version')
    )
    enabled = models.BooleanField(default=True, verbose_name=_('Enabled'))
    page_number = models.PositiveIntegerField(
        db_index=True, default=1, editable=False,
        verbose_name=_('Page number')
    )

    objects = DocumentPageManager()
    passthrough = models.Manager()

    class Meta:
        ordering = ('page_number',)
        verbose_name = _('Document page')
        verbose_name_plural = _('Document pages')

    def __str__(self):
        return self.get_label()

    @cached_property
    def cache_partition(self):
        partition, created = self.document_version.cache.partitions.get_or_create(
            name=self.uuid
        )
        return partition

    def delete(self, *args, **kwargs):
        self.cache_partition.delete()
        super(DocumentPage, self).delete(*args, **kwargs)

    def detect_orientation(self):
        with self.document_version.open() as file_object:
            converter = get_converter_class()(
                file_object=file_object,
                mime_type=self.document_version.mimetype
            )
            return converter.detect_orientation(
                page_number=self.page_number
            )

    @property
    def document(self):
        return self.document_version.document

    def generate_image(self, user=None, **kwargs):
        transformation_list = self.get_combined_transformation_list(user=user, **kwargs)
        combined_cache_filename = BaseTransformation.combine(transformation_list)

        # Check is transformed image is available
        logger.debug('transformations cache filename: %s', combined_cache_filename)

        if not setting_disable_transformed_image_cache.value and self.cache_partition.get_file(filename=combined_cache_filename):
            logger.debug(
                'transformations cache file "%s" found', combined_cache_filename
            )
        else:
            logger.debug(
                'transformations cache file "%s" not found', combined_cache_filename
            )
            image = self.get_image(transformations=transformation_list)
            with self.cache_partition.create_file(filename=combined_cache_filename) as file_object:
                file_object.write(image.getvalue())

        return combined_cache_filename

    def get_absolute_url(self):
        return reverse(
            viewname='documents:document_page_view', kwargs={
                'document_page_id': self.pk
            }
        )

    def get_api_image_url(self, *args, **kwargs):
        """
        Create an unique URL combining:
        - the page's image URL
        - the interactive argument
        - a hash from the server side and interactive transformations
        The purpose of this unique URL is to allow client side caching
        if document page images.
        """
        transformations_hash = BaseTransformation.combine(
            self.get_combined_transformation_list(*args, **kwargs)
        )

        kwargs.pop('transformations', None)

        final_url = furl()
        final_url.args = kwargs
        final_url.path = reverse(
            viewname='rest_api:documentpage-image', kwargs={
                'pk': self.document.pk, 'version_pk': self.document_version.pk,
                'page_pk': self.pk
            }
        )
        final_url.args['_hash'] = transformations_hash

        return final_url.tostr()

    def get_combined_transformation_list(self, user=None, *args, **kwargs):
        """
        Return a list of transformation containing the server side
        document page transformation as well as tranformations created
        from the arguments as transient interactive transformation.
        """
        # Convert arguments into transformations
        transformations = kwargs.get('transformations', [])

        # Set sensible defaults if the argument is not specified or if the
        # argument is None
        width = kwargs.get('width', setting_display_width.value) or setting_display_width.value
        height = kwargs.get('height', setting_display_height.value) or setting_display_height.value
        rotation = kwargs.get('rotation', DEFAULT_ROTATION) or DEFAULT_ROTATION
        zoom_level = kwargs.get('zoom', DEFAULT_ZOOM_LEVEL) or DEFAULT_ZOOM_LEVEL

        if zoom_level < setting_zoom_min_level.value:
            zoom_level = setting_zoom_min_level.value

        if zoom_level > setting_zoom_max_level.value:
            zoom_level = setting_zoom_max_level.value

        # Generate transformation hash
        transformation_list = []

        maximum_layer_order = kwargs.get('maximum_layer_order', None)

        # Stored transformations first
        for stored_transformation in LayerTransformation.objects.get_for_object(
            self, maximum_layer_order=maximum_layer_order, as_classes=True,
            user=user
        ):
            transformation_list.append(stored_transformation)

        # Interactive transformations second
        for transformation in transformations:
            transformation_list.append(transformation)

        if rotation:
            transformation_list.append(
                TransformationRotate(degrees=rotation)
            )

        if width:
            transformation_list.append(
                TransformationResize(width=width, height=height)
            )

        if zoom_level:
            transformation_list.append(TransformationZoom(percent=zoom_level))

        return transformation_list

    def get_image(self, transformations=None):
        cache_filename = 'base_image'
        logger.debug('Page cache filename: %s', cache_filename)

        cache_file = self.cache_partition.get_file(filename=cache_filename)

        if not setting_disable_base_image_cache.value and cache_file:
            logger.debug('Page cache file "%s" found', cache_filename)

            with cache_file.open() as file_object:
                converter = get_converter_class()(
                    file_object=file_object
                )

                converter.seek_page(page_number=0)

                # This code is also repeated below to allow using a context
                # manager with cache_file.open and close it automatically.
                # Apply runtime transformations
                for transformation in transformations:
                    converter.transform(transformation=transformation)

                return converter.get_page()
        else:
            logger.debug('Page cache file "%s" not found', cache_filename)

            try:
                with self.document_version.get_intermediate_file() as file_object:
                    converter = get_converter_class()(
                        file_object=file_object
                    )
                    converter.seek_page(page_number=self.page_number - 1)

                    page_image = converter.get_page()

                    # Since open "wb+" doesn't create files, create it explicitly
                    with self.cache_partition.create_file(filename=cache_filename) as file_object:
                        file_object.write(page_image.getvalue())

                    # Apply runtime transformations
                    for transformation in transformations:
                        converter.transform(transformation=transformation)

                    return converter.get_page()
            except Exception as exception:
                # Cleanup in case of error
                logger.error(
                    'Error creating page cache file "%s"; %s',
                    cache_filename, exception
                )
                raise

    @property
    def is_in_trash(self):
        return self.document.is_in_trash

    def get_label(self):
        return _(
            'Page %(page_num)d out of %(total_pages)d of %(document)s'
        ) % {
            'document': force_text(self.document),
            'page_num': self.page_number,
            'total_pages': self.document_version.pages_all.count()
        }
    get_label.short_description = _('Label')

    def natural_key(self):
        return (self.page_number, self.document_version.natural_key())
    natural_key.dependencies = ['documents.DocumentVersion']

    @property
    def siblings(self):
        return DocumentPage.objects.filter(
            document_version=self.document_version
        )

    @property
    def uuid(self):
        """
        Make cache UUID a mix of version ID and page ID to avoid using stale
        images
        """
        return '{}-{}'.format(self.document_version.uuid, self.pk)


class DocumentPageResult(DocumentPage):
    class Meta:
        ordering = ('document_version__document', 'page_number')
        proxy = True
        verbose_name = _('Document page')
        verbose_name_plural = _('Document pages')



from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.document_indexing.models import Index, IndexInstanceNode
from mayan.apps.document_indexing.tests.literals import TEST_INDEX_LABEL

from .literals import (
    TEST_TAG_INDEX_HAS_TAG, TEST_TAG_INDEX_NO_TAG, TEST_TAG_INDEX_NODE_TEMPLATE
)
from .mixins import TagTestMixin


class TagSignalIndexingTestCase(DocumentTestMixin, TagTestMixin, BaseTestCase):
    auto_upload_test_document = False

    def test_tag_indexing(self):
        self._create_test_tag()
        self.test_index = Index.objects.create(label=TEST_INDEX_LABEL)
        self.test_index.document_types.add(self.test_document_type)

        root = self.test_index.template_root
        self.test_index.node_templates.create(
            parent=root, expression=TEST_TAG_INDEX_NODE_TEMPLATE,
            link_documents=True
        )

        self._upload_test_document()

        self.assertTrue(
            self.test_document in IndexInstanceNode.objects.get(
                value=TEST_TAG_INDEX_NO_TAG
            ).documents.all()
        )

        self.test_tag.documents.add(self.test_document)

        self.assertTrue(
            self.test_document in IndexInstanceNode.objects.get(
                value=TEST_TAG_INDEX_HAS_TAG
            ).documents.all()
        )

        self.test_tag.delete()

        self.assertTrue(
            self.test_document in IndexInstanceNode.objects.get(
                value=TEST_TAG_INDEX_NO_TAG
            ).documents.all()
        )



import os

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

import mayan
from mayan.apps.smart_settings.classes import Namespace

from .literals import DEFAULT_COMMON_HOME_VIEW
from .setting_migrations import CommonSettingMigration

namespace = Namespace(
    label=_('Common'), migration_class=CommonSettingMigration,
    name='common', version='0002'
)

setting_auto_logging = namespace.add_setting(
    global_name='COMMON_AUTO_LOGGING',
    default=True,
    help_text=_('Automatically enable logging to all apps.')
)
settings_db_sync_task_delay = namespace.add_setting(
    global_name='COMMON_DB_SYNC_TASK_DELAY',
    default=2,
    help_text=_(
        'Time to delay background tasks that depend on a database commit to '
        'propagate.'
    )
)
setting_disabled_apps = namespace.add_setting(
    global_name='COMMON_DISABLED_APPS',
    default=settings.COMMON_DISABLED_APPS,
    help_text=_(
        'A list of strings designating all applications that are to be removed '
        'from the list normally installed by Mayan EDMS. Each string should be '
        'a dotted Python path to: an application configuration class (preferred), '
        'or a package containing an application.'
    ),
)
setting_extra_apps = namespace.add_setting(
    global_name='COMMON_EXTRA_APPS',
    default=settings.COMMON_EXTRA_APPS,
    help_text=_(
        'A list of strings designating all applications that are installed '
        'beyond those normally installed by Mayan EDMS. Each string should be '
        'a dotted Python path to: an application configuration class (preferred), '
        'or a package containing an application.'
    ),
)
setting_home_view = namespace.add_setting(
    global_name='COMMON_HOME_VIEW',
    default=DEFAULT_COMMON_HOME_VIEW, help_text=_(
        'Name of the view attached to the brand anchor in the main menu. '
        'This is also the view to which users will be redirected after '
        'log in.'
    ),
)
setting_paginate_by = namespace.add_setting(
    global_name='COMMON_PAGINATE_BY',
    default=40,
    help_text=_(
        'The number objects that will be displayed per page.'
    )
)
setting_production_error_logging = namespace.add_setting(
    global_name='COMMON_PRODUCTION_ERROR_LOGGING',
    default=False,
    help_text=_(
        'Enable error logging outside of the system error logging '
        'capabilities.'
    )
)
setting_production_error_log_path = namespace.add_setting(
    global_name='COMMON_PRODUCTION_ERROR_LOG_PATH',
    default=os.path.join(settings.MEDIA_ROOT, 'error.log'), help_text=_(
        'Path to the logfile that will track errors during production.'
    ),
    is_path=True
)
setting_project_title = namespace.add_setting(
    global_name='COMMON_PROJECT_TITLE',
    default=mayan.__title__, help_text=_(
        'Name to be displayed in the main menu.'
    ),
)
setting_project_url = namespace.add_setting(
    global_name='COMMON_PROJECT_URL',
    default=mayan.__website__, help_text=_(
        'URL of the installation or homepage of the project.'
    ),
)
setting_shared_storage = namespace.add_setting(
    global_name='COMMON_SHARED_STORAGE',
    default='django.core.files.storage.FileSystemStorage',
    help_text=_('A storage backend that all workers can use to share files.')
)
setting_shared_storage_arguments = namespace.add_setting(
    global_name='COMMON_SHARED_STORAGE_ARGUMENTS',
    default={'location': os.path.join(settings.MEDIA_ROOT, 'shared_files')}
)
setting_url_base_path = namespace.add_setting(
    global_name='COMMON_URL_BASE_PATH', default='',
    help_text=_('Base URL path to use for all views.')
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_fast, worker_medium

queue_sources = CeleryQueue(
    label=_('Sources'), name='sources', worker=worker_medium
)
queue_sources_periodic = CeleryQueue(
    label=_('Sources periodic'), name='sources_periodic', transient=True,
    worker=worker_medium
)
queue_sources_fast = CeleryQueue(
    label=_('Sources fast'), name='sources_fast', transient=True,
    worker=worker_fast
)

queue_sources_fast.add_task_type(
    label=_('Generate staging file image'),
    dotted_path='mayan.apps.sources.tasks.task_generate_staging_file_image'
)
queue_sources_periodic.add_task_type(
    label=_('Check interval source'),
    dotted_path='mayan.apps.sources.tasks.task_check_interval_source'
)
queue_sources.add_task_type(
    label=_('Handle upload'),
    dotted_path='mayan.apps.sources.tasks.task_source_handle_upload'
)
queue_sources.add_task_type(
    label=_('Upload document'),
    dotted_path='mayan.apps.sources.tasks.task_upload_document'
)



from ..models import FavoriteDocument
from ..permissions import permission_document_view

from .base import GenericDocumentViewTestCase
from .mixins import FavoriteDocumentsTestMixin


class FavoriteDocumentsTestCase(
    FavoriteDocumentsTestMixin, GenericDocumentViewTestCase
):

    def test_document_add_to_favorites_view_no_permission(self):
        response = self._request_document_add_to_favorites_view()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(FavoriteDocument.objects.count(), 0)

    def test_document_add_to_favorites_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_add_to_favorites_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(FavoriteDocument.objects.count(), 1)

    def test_document_list_favorites_view_no_permission(self):
        self._document_add_to_favorites()
        response = self._request_document_list_favorites()
        self.assertNotContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_list_favorites_view_with_access(self):
        self._document_add_to_favorites()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_list_favorites()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_remove_from_favorites_view_no_permission(self):
        self._document_add_to_favorites()
        response = self._request_document_remove_from_favorites()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(FavoriteDocument.objects.count(), 1)

    def test_document_remove_from_favorites_view_with_access(self):
        self._document_add_to_favorites()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )
        response = self._request_document_remove_from_favorites()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(FavoriteDocument.objects.count(), 0)



import logging

from django.contrib import messages
from django.template import RequestContext
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    MultipleObjectFormActionView, SingleObjectCreateView,
    SingleObjectDeleteView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.common.mixins import ExternalObjectMixin
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.models import Document
from mayan.apps.documents.views.document_views import DocumentListView

from .forms import CabinetListForm
from .icons import icon_cabinet
from .links import (
    link_document_cabinet_add, link_cabinet_child_add, link_cabinet_create
)
from .models import Cabinet
from .permissions import (
    permission_cabinet_add_document, permission_cabinet_create,
    permission_cabinet_delete, permission_cabinet_edit,
    permission_cabinet_view, permission_cabinet_remove_document
)
from .widgets import jstree_data

logger = logging.getLogger(name=__name__)


class CabinetCreateView(SingleObjectCreateView):
    fields = ('label',)
    model = Cabinet
    post_action_redirect = reverse_lazy(viewname='cabinets:cabinet_list')
    view_permission = permission_cabinet_create

    def get_extra_context(self):
        return {
            'title': _('Create cabinet'),
        }

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class CabinetChildAddView(ExternalObjectMixin, SingleObjectCreateView):
    fields = ('label',)
    external_object_class = Cabinet
    external_object_permission = permission_cabinet_edit
    external_object_pk_url_kwarg = 'cabinet_id'

    def get_extra_context(self):
        return {
            'title': _(
                'Add new level to: %s'
            ) % self.external_object.get_full_path(),
            'object': self.external_object
        }

    def get_instance_extra_data(self):
        return {
            'parent': self.external_object,
        }

    def get_queryset(self):
        return self.external_object.get_descendants()

    def get_save_extra_data(self):
        return {
            '_user': self.request.user
        }


class CabinetDeleteView(SingleObjectDeleteView):
    model = Cabinet
    object_permission = permission_cabinet_delete
    post_action_redirect = reverse_lazy(viewname='cabinets:cabinet_list')
    pk_url_kwarg = 'cabinet_id'

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Delete the cabinet: %s?') % self.object,
        }


class CabinetDetailView(ExternalObjectMixin, DocumentListView):
    external_object_class = Cabinet
    external_object_permission = permission_cabinet_view
    external_object_pk_url_kwarg = 'cabinet_id'
    template_name = 'cabinets/cabinet_details.html'

    def get_document_queryset(self):
        return self.external_object.documents.all()

    def get_extra_context(self, **kwargs):
        context = super(CabinetDetailView, self).get_extra_context(**kwargs)

        context.update(
            {
                'column_class': 'col-xs-12 col-sm-6 col-md-4 col-lg-3',
                'hide_links': True,
                'jstree_data': '\n'.join(
                    jstree_data(
                        node=self.external_object.get_root(),
                        selected_node=self.external_object
                    )
                ),
                'list_as_items': True,
                'no_results_icon': icon_cabinet,
                'no_results_main_link': link_cabinet_child_add.resolve(
                    context=RequestContext(
                        request=self.request, dict_={
                            'object': self.external_object
                        }
                    )
                ),
                'no_results_text': _(
                    'Cabinet levels can contain documents or other '
                    'cabinet sub levels. To add documents to a cabinet, '
                    'select the cabinet view of a document view.'
                ),
                'no_results_title': _('This cabinet level is empty'),
                'object': self.external_object,
                'title': _(
                    'Details of cabinet: %s'
                ) % self.external_object.get_full_path(),
            }
        )

        return context

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class CabinetEditView(SingleObjectEditView):
    fields = ('label',)
    model = Cabinet
    object_permission = permission_cabinet_edit
    post_action_redirect = reverse_lazy(viewname='cabinets:cabinet_list')
    pk_url_kwarg = 'cabinet_id'

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _('Edit cabinet: %s') % self.object,
        }

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class CabinetListView(SingleObjectListView):
    object_permission = permission_cabinet_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'title': _('Cabinets'),
            'no_results_icon': icon_cabinet,
            'no_results_main_link': link_cabinet_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Cabinets are a multi-level method to organize '
                'documents. Each cabinet can contain documents as '
                'well as other sub level cabinets.'
            ),
            'no_results_title': _('No cabinets available'),
        }

    def get_source_queryset(self):
        # Add explicit ordering of root nodes since the queryset returned
        # is not affected by the model's order Meta option.
        return Cabinet.objects.root_nodes().order_by('label')


class DocumentCabinetListView(ExternalObjectMixin, CabinetListView):
    external_object_class = Document
    external_object_permission = permission_document_view
    external_object_pk_url_kwarg = 'document_id'

    def get_extra_context(self):
        return {
            'hide_link': True,
            'no_results_icon': icon_cabinet,
            'no_results_main_link': link_document_cabinet_add.resolve(
                context=RequestContext(
                    request=self.request, dict_={
                        'object': self.external_object
                    }
                )
            ),
            'no_results_text': _(
                'Documents can be added to many cabinets.'
            ),
            'no_results_title': _(
                'This document is not in any cabinet'
            ),
            'object': self.external_object,
            'title': _(
                'Cabinets containing document: %s'
            ) % self.external_object,
        }

    def get_source_queryset(self):
        return self.external_object.get_cabinets(
            permission=permission_cabinet_view, user=self.request.user
        )


class DocumentAddToCabinetView(MultipleObjectFormActionView):
    form_class = CabinetListForm
    model = Document
    object_permission = permission_cabinet_add_document
    pk_url_kwarg = 'document_id'
    success_message = _(
        'Add to cabinet request performed on %(count)d document.'
    )
    success_message_plural = _(
        'Add to cabinet request performed on %(count)d documents.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'submit_label': _('Add'),
            'title': ungettext(
                singular='Add %(count)d document to cabinets',
                plural='Add %(count)d documents to cabinets',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Add document "%s" to cabinets'
                    ) % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        queryset = self.object_list
        result = {
            'help_text': _(
                'Cabinets to which the selected documents will be added.'
            ),
            'permission': permission_cabinet_add_document,
            'queryset': Cabinet.objects.all(),
            'user': self.request.user
        }

        if queryset.count() == 1:
            result.update(
                {
                    'queryset': Cabinet.objects.exclude(
                        pk__in=queryset.first().cabinets.all()
                    )
                }
            )

        return result

    def object_action(self, form, instance):
        cabinet_membership = instance.cabinets.all()

        for cabinet in form.cleaned_data['cabinets']:
            AccessControlList.objects.check_access(
                obj=cabinet, permissions=(permission_cabinet_add_document,),
                user=self.request.user
            )
            if cabinet in cabinet_membership:
                messages.warning(
                    message=_(
                        'Document: %(document)s is already in '
                        'cabinet: %(cabinet)s.'
                    ) % {
                        'document': instance, 'cabinet': cabinet
                    }, request=self.request
                )
            else:
                cabinet.document_add(
                    document=instance, _user=self.request.user
                )
                messages.success(
                    message=_(
                        'Document: %(document)s added to cabinet: '
                        '%(cabinet)s successfully.'
                    ) % {
                        'document': instance, 'cabinet': cabinet
                    }, request=self.request
                )


class DocumentRemoveFromCabinetView(MultipleObjectFormActionView):
    form_class = CabinetListForm
    model = Document
    object_permission = permission_cabinet_remove_document
    pk_url_kwarg = 'document_id'
    success_message = _(
        'Remove from cabinet request performed on %(count)d document.'
    )
    success_message_plural = _(
        'Remove from cabinet request performed on %(count)d documents.'
    )

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'submit_label': _('Remove'),
            'title': ungettext(
                singular='Remove %(count)d document from cabinets',
                plural='Remove %(count)d documents from cabinets',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Remove document "%s" from cabinets'
                    ) % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        queryset = self.object_list
        result = {
            'help_text': _(
                'Cabinets from which the selected documents will be removed.'
            ),
            'permission': permission_cabinet_remove_document,
            'queryset': Cabinet.objects.all(),
            'user': self.request.user
        }

        if queryset.count() == 1:
            result.update(
                {
                    'queryset': queryset.first().cabinets.all()
                }
            )

        return result

    def object_action(self, form, instance):
        cabinet_membership = instance.cabinets.all()

        for cabinet in form.cleaned_data['cabinets']:
            AccessControlList.objects.check_access(
                obj=cabinet, permissions=(permission_cabinet_remove_document,),
                user=self.request.user
            )

            if cabinet not in cabinet_membership:
                messages.warning(
                    message=_(
                        'Document: %(document)s is not in cabinet: '
                        '%(cabinet)s.'
                    ) % {
                        'document': instance, 'cabinet': cabinet
                    }, request=self.request
                )
            else:
                cabinet.document_remove(
                    document=instance, _user=self.request.user
                )
                messages.success(
                    message=_(
                        'Document: %(document)s removed from cabinet: '
                        '%(cabinet)s.'
                    ) % {
                        'document': instance, 'cabinet': cabinet
                    }, request=self.request
                )



from django.apps import apps
from django.contrib.auth import get_user_model
from django.db import transaction
from django.shortcuts import reverse

from .events import (
    event_group_created, event_group_edited, event_user_created,
    event_user_edited
)
from .permissions import permission_group_view, permission_user_view
from .querysets import get_user_queryset


def get_method_group_save():
    Group = apps.get_model(app_label='auth', model_name='Group')
    group_save_original = Group.save

    def method_group_save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)

        with transaction.atomic():
            is_new = not self.pk
            group_save_original(self, *args, **kwargs)
            if is_new:
                event_group_created.commit(
                    actor=_user, target=self
                )
            else:
                event_group_edited.commit(
                    actor=_user, target=self
                )

    return method_group_save


def method_group_get_users(self, user, permission=permission_user_view):
    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )

    return AccessControlList.objects.restrict_queryset(
        permission=permission, queryset=get_user_queryset().filter(
            id__in=self.user_set.all()
        ), user=user
    )


def method_group_users_add(self, queryset, _user):
    with transaction.atomic():
        event_group_edited.commit(
            actor=_user, target=self
        )
        for user in queryset:
            self.user_set.add(user)
            event_user_edited.commit(
                actor=_user, target=user
            )


def method_group_users_remove(self, queryset, _user):
    with transaction.atomic():
        event_group_edited.commit(
            actor=_user, target=self
        )
        for user in queryset:
            self.user_set.remove(user)
            event_user_edited.commit(
                actor=_user, target=user
            )


def method_user_get_absolute_url(self):
    return reverse(
        viewname='user_management:user_details', kwargs={'user_id': self.pk}
    )


def method_user_get_groups(self, user, permission=permission_group_view):
    AccessControlList = apps.get_model(
        app_label='acls', model_name='AccessControlList'
    )

    return AccessControlList.objects.restrict_queryset(
        permission=permission, queryset=self.groups.all(), user=user
    )


def method_user_groups_add(self, queryset, _user):
    with transaction.atomic():
        event_user_edited.commit(
            actor=_user, target=self
        )
        for group in queryset:
            self.groups.add(group)
            event_group_edited.commit(
                actor=_user, target=group
            )


def method_user_groups_remove(self, queryset, _user):
    with transaction.atomic():
        event_user_edited.commit(
            actor=_user, target=self
        )
        for group in queryset:
            self.groups.remove(group)
            event_group_edited.commit(
                actor=_user, target=group
            )


def get_method_user_save():
    user_save_original = get_user_model().save

    def method_user_save(self, *args, **kwargs):
        _user = kwargs.pop('_user', None)

        with transaction.atomic():
            is_new = not self.pk
            user_save_original(self, *args, **kwargs)
            if is_new:
                event_user_created.commit(
                    actor=_user, target=self
                )
            else:
                event_user_edited.commit(
                    actor=_user, target=self
                )

    return method_user_save



from django.test import override_settings

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.documents.tests.literals import TEST_HYBRID_DOCUMENT

from ..models import DocumentPageContent
from ..permissions import (
    permission_content_view, permission_document_type_parsing_setup,
    permission_parse_document
)
from ..utils import get_instance_content

from .literals import TEST_DOCUMENT_CONTENT
from .mixins import (
    DocumentContentToolsViewsTestMixin, DocumentContentViewTestMixin,
    DocumentTypeContentViewsTestMixin
)


@override_settings(DOCUMENT_PARSING_AUTO_PARSING=True)
class DocumentContentViewsTestCase(
    DocumentContentViewTestMixin, GenericDocumentViewTestCase
):
    _skip_file_descriptor_test = True

    # Ensure we use a PDF file
    test_document_filename = TEST_HYBRID_DOCUMENT

    def test_document_content_view_no_permissions(self):
        response = self._request_test_document_content_view()
        self.assertEqual(response.status_code, 404)

    def test_document_content_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_content_view
        )

        response = self._request_test_document_content_view()
        self.assertContains(
            response=response, text=TEST_DOCUMENT_CONTENT, status_code=200
        )

    def test_document_content_delete_view_no_permissions(self):
        response = self._request_test_document_content_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            DocumentPageContent.objects.filter(
                document_page=self.test_document.pages.first()
            ).exists()
        )

    def test_document_content_delete_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_parse_document
        )

        response = self._request_test_document_content_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            DocumentPageContent.objects.filter(
                document_page=self.test_document.pages.first()
            ).exists()
        )

    def test_document_page_content_view_no_permissions(self):
        response = self._request_test_document_page_content_view()
        self.assertEqual(response.status_code, 404)

    def test_document_page_content_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_content_view
        )

        response = self._request_test_document_page_content_view()
        self.assertContains(
            response=response, text=TEST_DOCUMENT_CONTENT, status_code=200
        )

    def test_document_parsing_download_view_no_permission(self):
        response = self._request_test_document_content_download_view()
        self.assertEqual(response.status_code, 404)

    def test_document_parsing_download_view_with_access(self):
        self.expected_content_types = ('text/html; charset=utf-8',)
        self.grant_access(
            obj=self.test_document, permission=permission_content_view
        )

        response = self._request_test_document_content_download_view()
        self.assertEqual(response.status_code, 200)

        self.assert_download_response(
            response=response, content=(
                ''.join(get_instance_content(document=self.test_document))
            ),
        )

    def test_document_parsing_error_list_view_no_permission(self):
        response = self._request_test_document_parsing_error_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_parsing_error_list_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_parse_document
        )

        response = self._request_test_document_parsing_error_list_view()
        self.assertEqual(response.status_code, 200)


class DocumentTypeContentViewsTestCase(
    DocumentTypeContentViewsTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_document_type_parsing_settings_view_no_permission(self):
        response = self._request_test_document_type_parsing_settings()
        self.assertEqual(response.status_code, 404)

    def test_document_type_parsing_settings_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_parsing_setup
        )

        response = self._request_test_document_type_parsing_settings()
        self.assertEqual(response.status_code, 200)


class DocumentContentToolsViewsTestCase(
    DocumentContentToolsViewsTestMixin, GenericDocumentViewTestCase
):
    _skip_file_descriptor_test = True
    auto_upload_test_document = False

    # Ensure we use a PDF file
    test_document_filename = TEST_HYBRID_DOCUMENT

    def _get_instance_content(self):
        return ''.join(
            list(get_instance_content(document=self.test_document))
        )

    def test_document_parsing_error_list_view_no_permission(self):
        response = self._request_document_parsing_error_list_view()
        self.assertEqual(response.status_code, 403)

    def test_document_parsing_error_list_view_with_permission(self):
        self.grant_permission(permission=permission_parse_document)

        response = self._request_document_parsing_error_list_view()
        self.assertEqual(response.status_code, 200)

    def test_document_parsing_tool_view_no_permission(self):
        self._upload_test_document()

        response = self._request_document_parsing_tool_view()
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(
            response=response, status_code=200,
            text=self.test_document_type.label
        )

        self.assertNotEqual(
            self._get_instance_content(), TEST_DOCUMENT_CONTENT
        )

    def test_document_parsing_tool_view_with_permission(self):
        self.grant_permission(permission=permission_parse_document)

        self._upload_test_document()

        response = self._request_document_parsing_tool_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._get_instance_content(), TEST_DOCUMENT_CONTENT
        )



from django.test import override_settings

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..permissions import (
    permission_document_type_file_metadata_setup,
    permission_file_metadata_submit, permission_file_metadata_view
)

from .literals import TEST_FILE_METADATA_KEY


class FileMetadataViewsTestMixin(object):
    def _request_document_version_driver_list_view(self):
        return self.get(
            viewname='file_metadata:document_driver_list',
            kwargs={'document_id': self.test_document.pk}
        )

    def _request_document_version_file_metadata_list_view(self):
        return self.get(
            viewname='file_metadata:document_version_driver_file_metadata_list',
            kwargs={
                'document_version_driver_id': self.test_driver.pk
            }
        )

    def _request_document_submit_view(self):
        return self.post(
            viewname='file_metadata:document_submit',
            kwargs={'document_id': self.test_document.pk}
        )

    def _request_multiple_document_submit_view(self):
        return self.post(
            viewname='file_metadata:document_multiple_submit',
            data={
                'id_list': self.test_document.pk,
            }
        )


@override_settings(FILE_METADATA_AUTO_PROCESS=True)
class FileMetadataViewsTestCase(
    FileMetadataViewsTestMixin, GenericDocumentViewTestCase
):
    def setUp(self):
        super(FileMetadataViewsTestCase, self).setUp()
        self.test_driver = self.test_document.latest_version.file_metadata_drivers.first()

    def test_document_version_driver_list_view_no_permission(self):
        response = self._request_document_version_driver_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_version_driver_list_view_with_access(self):
        self.grant_access(
            permission=permission_file_metadata_view, obj=self.test_document
        )

        response = self._request_document_version_driver_list_view()
        self.assertContains(
            response=response, text=self.test_document.label, status_code=200
        )

    def test_document_version_file_metadata_list_view_no_permission(self):
        response = self._request_document_version_file_metadata_list_view()
        self.assertNotContains(
            response=response, text=TEST_FILE_METADATA_KEY, status_code=404
        )

    def test_document_version_file_metadata_list_view_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_file_metadata_view
        )

        response = self._request_document_version_file_metadata_list_view()
        self.assertContains(
            response=response, text=TEST_FILE_METADATA_KEY, status_code=200
        )

    def test_document_submit_view_no_permission(self):
        self.test_document.latest_version.file_metadata_drivers.all().delete()

        response = self._request_document_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 0
        )

    def test_document_submit_view_with_access(self):
        self.test_document.latest_version.file_metadata_drivers.all().delete()
        self.grant_access(
            permission=permission_file_metadata_submit, obj=self.test_document
        )

        response = self._request_document_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 1
        )

    def test_multiple_document_submit_view_no_permission(self):
        self.test_document.latest_version.file_metadata_drivers.all().delete()

        response = self._request_multiple_document_submit_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 0
        )

    def test_multiple_document_submit_view_with_access(self):
        self.test_document.latest_version.file_metadata_drivers.all().delete()
        self.grant_access(
            permission=permission_file_metadata_submit, obj=self.test_document
        )

        response = self._request_multiple_document_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 1
        )


class DocumentTypeViewsTestMixin(object):
    def _request_document_type_settings_view(self):
        return self.get(
            viewname='file_metadata:document_type_settings',
            kwargs={'document_type_id': self.test_document.document_type.pk}
        )

    def _request_document_type_submit_view(self):
        return self.post(
            viewname='file_metadata:document_type_submit', data={
                'document_type': self.test_document_type.pk,
            }
        )


class DocumentTypeViewsTestCase(
    DocumentTypeViewsTestMixin, GenericDocumentViewTestCase
):
    def test_document_type_settings_view_no_permission(self):
        response = self._request_document_type_settings_view()
        self.assertEqual(response.status_code, 404)

    def test_document_type_settings_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type,
            permission=permission_document_type_file_metadata_setup
        )

        response = self._request_document_type_settings_view()
        self.assertEqual(response.status_code, 200)

    def test_document_type_submit_view_no_permission(self):
        response = self._request_document_type_submit_view()
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 0
        )

    def test_document_type_submit_view_with_access(self):
        self.grant_access(
            obj=self.test_document_type, permission=permission_file_metadata_submit
        )

        response = self._request_document_type_submit_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.latest_version.file_metadata_drivers.count(), 1
        )



from actstream.models import Action

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.tests.mixins import DocumentTestMixin

from ..permissions import (
    permission_document_indexing_create, permission_document_indexing_edit,
)

from ..events import event_index_template_created, event_index_template_edited

from .mixins import IndexTestMixin, IndexViewTestMixin


class IndexTemplateEventsTestCase(DocumentTestMixin, IndexTestMixin, IndexViewTestMixin, GenericViewTestCase):
    auto_upload_test_document = False

    def test_index_template_create_event(self):
        Action.objects.all().delete()

        self.grant_permission(
            permission=permission_document_indexing_create
        )
        response = self._request_test_index_create_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.actor, self._test_case_user)
        self.assertEqual(action.target, self.test_index)
        self.assertEqual(action.verb, event_index_template_created.id)

    def test_index_template_edit_event(self):
        self._create_test_index()

        self.grant_access(
            obj=self.test_index, permission=permission_document_indexing_edit
        )
        Action.objects.all().delete()

        response = self._request_test_index_edit_view()
        self.assertEqual(response.status_code, 302)

        action = Action.objects.last()

        self.assertEqual(action.actor, self._test_case_user)
        self.assertEqual(action.target, self.test_index)
        self.assertEqual(action.verb, event_index_template_edited.id)



from django.apps import apps
from django.conf import settings
from django.template import Library
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

app_templates_cache = {}
register = Library()


@register.simple_tag
def appearance_app_templates(template_name):
    result = []

    for app in apps.get_app_configs():
        template_id = '{}.{}'.format(app.label, template_name)

        if settings.DEBUG or template_id not in app_templates_cache:
            try:
                app_templates_cache[template_id] = get_template(
                    '{}/app/{}.html'.format(app.label, template_name)
                ).render()
            except TemplateDoesNotExist:
                """Non fatal"""
                app_templates_cache[template_id] = ''

        result.append(app_templates_cache[template_id])

    return mark_safe('\n'.join(result))


@register.filter
def appearance_get_choice_value(field):
    try:
        return dict(field.field.choices)[field.value()]
    except TypeError:
        return ', '.join([subwidget.data['label'] for subwidget in field.subwidgets if subwidget.data['selected']])
    except KeyError:
        return _('None')


@register.filter
def appearance_get_form_media_js(form):
    return [form.media.absolute_path(path) for path in form.media._js]


@register.simple_tag
def appearance_get_icon(icon_path):
    return import_string(dotted_path=icon_path).render()


@register.simple_tag
def appearance_icon_render(icon_class, enable_shadow=False):
    return icon_class.render(extra_context={'enable_shadow': enable_shadow})


@register.filter
def appearance_object_list_count(object_list):
    try:
        return object_list.count()
    except TypeError:
        return len(object_list)



import uuid

import django.core.files.storage
from django.db import migrations, models
from django.utils.encoding import force_text


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0041_auto_20170823_1855'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documentversion',
            name='file',
            field=models.FileField(
                storage=django.core.files.storage.FileSystemStorage(
                    location=b'mayan/media/document_storage'
                ), upload_to=force_text(uuid.uuid4()), verbose_name='File'
            ),
        ),
    ]



import uuid

from django.db import models, migrations
from django.core.files.storage import FileSystemStorage
from django.utils.encoding import force_text


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0002_auto_20150608_1902'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='uuid',
            field=models.CharField(
                default=force_text(uuid.uuid4()), max_length=48,
                editable=False
            ),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='documentversion',
            name='file',
            field=models.FileField(
                upload_to=force_text(uuid.uuid4()),
                storage=FileSystemStorage(),
                verbose_name='File'
            ),
            preserve_default=True,
        ),
    ]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.events.classes import EventTypeNamespace

namespace = EventTypeNamespace(label=_('OCR'), name='ocr')

event_ocr_document_content_deleted = namespace.add_event_type(
    label=_('Document OCR content deleted'),
    name='document_content_deleted'
)
event_ocr_document_version_submit = namespace.add_event_type(
    label=_('Document version submitted for OCR'),
    name='document_version_submit'
)
event_ocr_document_version_finish = namespace.add_event_type(
    label=_('Document version OCR finished'),
    name='document_version_finish'
)



import platform

CONVERTER_OFFICE_FILE_MIMETYPES = (
    'application/msword',
    'application/mswrite',
    'application/mspowerpoint',
    'application/msexcel',
    'application/pgp-keys',
    'application/vnd.ms-excel',
    'application/vnd.ms-excel.addin.macroEnabled.12',
    'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
    'application/vnd.ms-powerpoint',
    'application/vnd.oasis.opendocument.chart',
    'application/vnd.oasis.opendocument.chart-template',
    'application/vnd.oasis.opendocument.formula',
    'application/vnd.oasis.opendocument.formula-template',
    'application/vnd.oasis.opendocument.graphics',
    'application/vnd.oasis.opendocument.graphics-template',
    'application/vnd.oasis.opendocument.image',
    'application/vnd.oasis.opendocument.image-template',
    'application/vnd.oasis.opendocument.presentation',
    'application/vnd.oasis.opendocument.presentation-template',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
    'application/vnd.openxmlformats-officedocument.presentationml.template',
    'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'application/vnd.openxmlformats-officedocument.presentationml.slide',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
    'application/vnd.oasis.opendocument.spreadsheet',
    'application/vnd.oasis.opendocument.spreadsheet-template',
    'application/vnd.oasis.opendocument.text',
    'application/vnd.oasis.opendocument.text-master',
    'application/vnd.oasis.opendocument.text-template',
    'application/vnd.oasis.opendocument.text-web',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'application/vnd.ms-office',
    'application/xml',
    'text/x-c',
    'text/x-c++',
    'text/x-pascal',
    'text/x-msdos-batch',
    'text/x-python',
    'text/x-shellscript',
    'text/plain',
    'text/rtf',
)

if platform.system() in ('FreeBSD', 'OpenBSD', 'Darwin'):
    DEFAULT_LIBREOFFICE_PATH = '/usr/local/bin/libreoffice'
    DEFAULT_PDFINFO_PATH = '/usr/local/bin/pdfinfo'
    DEFAULT_PDFTOPPM_PATH = '/usr/local/bin/pdftoppm'
else:
    DEFAULT_LIBREOFFICE_PATH = '/usr/bin/libreoffice'
    DEFAULT_PDFINFO_PATH = '/usr/bin/pdfinfo'
    DEFAULT_PDFTOPPM_PATH = '/usr/bin/pdftoppm'

DEFAULT_ZOOM_LEVEL = 100
DEFAULT_ROTATION = 0
DEFAULT_PAGE_NUMBER = 1
DEFAULT_PILLOW_FORMAT = 'JPEG'
DEFAULT_PILLOW_MAXIMUM_IMAGE_PIXELS = 89478485  # Upstream default as of v6.2.1 (2019-01-16)

DEFAULT_PDFTOPPM_DPI = 300
DEFAULT_PDFTOPPM_FORMAT = 'jpeg'  # Possible values jpeg, png, tiff



from django.contrib import messages
from django.http import Http404
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.generics import ConfirmView, SimpleView, SingleObjectListView

from .classes import Statistic, StatisticNamespace
from .permissions import permission_statistics_view
from .tasks import task_execute_statistic


class NamespaceListView(SingleObjectListView):
    extra_context = {
        'hide_link': True,
        'title': _('Statistics namespaces'),
    }
    template_name = 'appearance/generic_list.html'
    view_permission = permission_statistics_view

    def get_source_queryset(self):
        return StatisticNamespace.get_all()


class NamespaceDetailView(SingleObjectListView):
    view_permission = permission_statistics_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'object': self.get_namespace(),
            'title': _('Namespace details for: %s') % self.get_namespace(),
        }

    def get_namespace(self):
        return StatisticNamespace.get(slug=self.kwargs['slug'])

    def get_source_queryset(self):
        return self.get_namespace().statistics


class StatisticDetailView(SimpleView):
    view_permission = permission_statistics_view

    def get_extra_context(self):
        obj = self.get_object()

        return {
            'chart_data': obj.get_chart_data(),
            'namespace': obj.namespace,
            'navigation_object_list': ('namespace', 'object'),
            'no_data': not obj.get_results_data()['series'],
            'object': obj,
            'title': _('Results for: %s') % obj,
        }

    def get_object(self):
        try:
            return Statistic.get(self.kwargs['slug'])
        except KeyError:
            raise Http404(_('Statistic "%s" not found.') % self.kwargs['slug'])

    def get_template_names(self):
        return (self.get_object().renderer.template_name,)


class StatisticQueueView(ConfirmView):
    view_permission = permission_statistics_view

    def get_extra_context(self):
        obj = self.get_object()
        return {
            'namespace': obj.namespace,
            'object': obj,
            # Translators: This text is asking users if they want to queue
            # (to send to the queue) a statistic for it to be update ahead
            # of schedule
            'title': _(
                'Queue statistic "%s" to be updated?'
            ) % obj,
        }

    def get_object(self):
        try:
            return Statistic.get(slug=self.kwargs['slug'])
        except KeyError:
            raise Http404(_('Statistic "%s" not found.') % self.kwargs['slug'])

    def view_action(self):
        task_execute_statistic.delay(slug=self.get_object().slug)
        messages.success(
            message=_(
                'Statistic "%s" queued successfully for update.'
            ) % self.get_object().label, request=self.request
        )
